<?php
namespace Service;

use \Util\Exception;

class Account {

	protected $entityManager;
	protected $precoAPI;
	protected $util;
	protected $vendaAPI;
	protected $tableService;
    protected $pedidoService;
	protected $waiterMessage;
	protected $billService;
	protected $impressaoService;
	protected $transactionsService;
	protected $date;
	protected $registerService;
	protected $caixaAPI;
    protected $extratocons;
 	protected $parametrosAPI;
    protected $utilAPI;
    protected $consumidorAPI;
    protected $consumidorService;
	protected $fidelidadeUDSAPI;
	protected $promocaoService;
	protected $fidelidadeOnline;

	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Odhen\API\Service\Preco $precoAPI,
		\Util\Util $util,
		\Odhen\API\Service\Venda $vendaAPI,
		\Service\Table $tableService,
        \Service\Pedido $pedidoService,
		\Util\WaiterMessage $waiterMessage,
		\Service\Bill $billService,
		\Service\Impressao $impressaoService,
		\Service\Transactions $transactionsService,
		\Util\Date $date,
		\Service\Register $registerService,
		\Odhen\API\Service\Caixa $caixaAPI,
        \Odhen\API\Service\Extratocons $extratocons,
 		\Odhen\API\Service\Parametros $parametrosAPI,
        \Odhen\API\Util\Util $utilAPI,
        \Odhen\API\Service\Consumidor $consumidorAPI,
        \Service\Consumer $consumidorService,
		\Odhen\API\Service\FidelidadeUDS $fidelidadeUDSAPI,
		\Service\Promocao $promocaoService,
		\Odhen\API\Service\FidelidadeOnline $fidelidadeOnline
	){
		$this->entityManager       = $entityManager;
		$this->precoAPI            = $precoAPI;
		$this->util                = $util;
		$this->vendaAPI            = $vendaAPI;
		$this->tableService        = $tableService;
        $this->pedidoService       = $pedidoService;
		$this->waiterMessage       = $waiterMessage;
		$this->billService         = $billService;
		$this->impressaoService    = $impressaoService;
		$this->transactionsService = $transactionsService;
		$this->date                = $date;
		$this->registerService 	   = $registerService;
		$this->caixaAPI   		   = $caixaAPI;
        $this->extratocons         = $extratocons;
		$this->parametrosAPI       = $parametrosAPI;
        $this->utilAPI             = $utilAPI;
        $this->consumidorAPI	   = $consumidorAPI;
        $this->consumidorService   = $consumidorService;
        $this->fidelidadeUDSAPI    = $fidelidadeUDSAPI;
		$this->promocaoService    = $promocaoService;
		$this->fidelidadeOnline    = $fidelidadeOnline;
	}

    public function arrayUDSPrePedido(
        $paramsUpdate, $ITEMVENDA
    ){
        $arrProdutos = array();
        foreach ($ITEMVENDA as $product){
            $productArray = array(
                "idProduto" => $product["CDPRODUTO"],
				"descProduto" => $product["NMPRODUTO"],
                "quantidade" => $product["QTPRODCOMVEN"],
                "valor" => array(
					"unitario" => round($product["VRPRECCOMVEN"], 2),
                    "acrescimo" => round($product["VRACRCOMVEN"]),
                    "desconto" => round($product["VRDESCCOMVEN"])
                )
            );
            array_push($arrProdutos, array( "data"=>$productArray) );
        }

        return array(
            "idEstabelecimento" => $paramsUpdate['CDFILIAL'],
            "codigoLoja" => $paramsUpdate['CDLOJA'],
            "idInterface" => $paramsUpdate['CDCAIXA'],
            "tipoInterface" => 1,
            "idPedido" =>  $paramsUpdate['CDFILIAL'].$paramsUpdate['NRVENDAREST'].$paramsUpdate['NRCOMANDA'],
            "conteudoPedido" =>$arrProdutos,
            "statusPedido" => 0,
            "usarCashback" => ( $paramsUpdate['VRDESCCASHUDS'] > 0 ? 'true' : 'false' ),
            "cpfCli" => $paramsUpdate['NRCPFFIDUDS'],
			"timestamp" => (new \DateTime())->format('Y-m-d')."T".(new \DateTime())->format('H:i:s'),
            "atualizado" => 'false'
        );
	}

	public function updateTableData($chave, $tableData, $CDCLIENTE, $CDCONSUMIDOR){
		$session = $this->util->getSessionVars($chave);
		foreach ($tableData as $table) {
			$cashback = null;
			$paramsUpdate = $this->formatUpdateTableData($session['CDFILIAL'], $table, $CDCLIENTE, $CDCONSUMIDOR, null, null, null, $cashback, null, null);
			$this->entityManager->getConnection()->executeQuery("UPDATE_COMANDAVEN_DESCFID", $paramsUpdate);

			$this->entityManager->getConnection()->executeQuery("UPDATE_VENDAREST", $paramsUpdate);
		}
	}

	public function formatUpdateTableData($CDFILIAL, $table, $CDCLIENTE, $CDCONSUMIDOR, $CPF4ALL, $CODPROMO, $DESCONTOFIDEL, $cashback, $TELCRMBONUS, $VRDESCCRMBONUS){
		$arrRetorno = array(
			'CDFILIAL'        => $CDFILIAL,
			'NRVENDAREST'     => $table['NRVENDAREST'],
			'NRCOMANDA'       => $table['NRCOMANDA'],
			'NRCPFFID4ALL'    => $CPF4ALL,
            'CDPROMOFID4ALL'  => $CODPROMO,
            'VRDESCPROMO4ALL' => $DESCONTOFIDEL,
            'VRDESCCASH4ALL'  => $cashback,
            'CDCLIENTE'       => $CDCLIENTE,
			'CDCONSUMIDOR'    => $CDCONSUMIDOR,
			'TELCRMBONUS'     => $TELCRMBONUS,
			'VRDESCCRMBONUS'  => $VRDESCCRMBONUS
		);
	
		if( empty($CPF4ALL) && empty($TELCRMBONUS) ){
			$arrRetorno['NRCPFFID4ALL']    = null;
			$arrRetorno['VRDESCPROMO4ALL'] = null;
			$arrRetorno['VRDESCCASH4ALL']  = null;
			$arrRetorno['TELCRMBONUS']     = null;
			$arrRetorno['VRDESCCRMBONUS']   = null;
		}
		if(!empty($TELCRMBONUS)){
			$arrRetorno['NRCPFFID4ALL']    = null;
			$arrRetorno['VRDESCPROMO4ALL'] = null;
			$arrRetorno['VRDESCCASH4ALL']  = null;
		}
		return $arrRetorno;
	}

	public function changeClientConsumer($chave, $NRVENDAREST, $NRCOMANDA, $positions, $CDCLIENTE, $CDCONSUMIDOR){

		$session    = $this->util->getSessionVars($chave);
        $this->checkClientConsumer($session['CDFILIAL'], $CDCLIENTE, $CDCONSUMIDOR);

		$result = array(
			'status'  => false,
			'message' => '',
			'data'    => array()
		);
		$connection = false;
		$positions  = $this->tableService->definePosition($positions); // ajusta posições com 0 a esquerda
		$CDFILIAL   = $session['CDFILIAL'];
        $CDLOJA     = $session['CDLOJA'];
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$connection = true;
		} catch (\Exception $e) {
			Exception::logException($e);
			$result['message'] = $this->waiterMessage->getMessage('265');
		}

		if ($connection){
			if ($session['IDMODULO'] !== 'C'){
				$tableData = $this->tableService->dadosMesa($CDFILIAL, $session['CDLOJA'], $NRCOMANDA, $NRVENDAREST);
                // Se tiver marcado, não altera o cliente/consumidor - mantêm os mesmos e remove o desconto fidelidade para recalcular o preço.
				$positions = array_column($positions, null, 'NRLUGARMESA');

                $params = array(
                    'CDFILIAL' => $session['CDFILIAL'],
                    'NRVENDAREST' => $NRVENDAREST
                );
                $valTrans = $this->entityManager->getConnection()->fetchAll("SQL_BUSCA_TRANSACOES_MOVCAIXADLV", $params);
                if (!empty($valTrans)){
                    return array('status' => false, 'message' => $this->waiterMessage->getMessage('462'));
                }

				$modifyTablePosition = $this->tableService->modifyTablePosition($chave, $NRVENDAREST, $positions);
				if ($modifyTablePosition['status']){
					if (empty($positions)){
						$CDCLIENTE = $CDCLIENTE ? $CDCLIENTE : $session['CDCLIENTE'];
						// cliente e consumidor por mesa. CDCLIENTE e CDCONSUMIDOR é mantido
						$this->updateTableData($chave, array($tableData), $CDCLIENTE, $CDCONSUMIDOR);
					} else {
						// cliente e consumidor por posição. CDCLIENTE e CDCONSUMIDOR salvo na mesa é utilizado como parâmetro quando a posição não for setada na recalcProductPrice
						$CDCLIENTE    = $tableData['CDCLIENTE'];
						$CDCONSUMIDOR = $tableData['CDCONSUMIDOR'];
					}
					// busca junção de mesas, se houver
					$tableGroup = $this->tableService->getTablesFromTableGrouping($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $tableData['NRMESA'], $session['NRORG'], $session['IDMODULO']);
					// busca itens a partir da NRVENDAREST, NRCOMANDA e posições
					$arrayItens = $this->vendaAPI->buscaItensPedidos($CDFILIAL, $CDLOJA, $tableGroup, array_column($positions, 'NRLUGARMESA'));
					$arrayItens = $this->tableService->removeCancelItens($arrayItens);
					// busca preços e executa update
					$recalcResult = $this->tableService->recalcProductPrice($CDFILIAL, $session['CDLOJA'], $arrayItens, $positions, $CDCLIENTE, $CDCONSUMIDOR, false);
					if ($recalcResult['status'] && $tableData['VRDESCFID']){
						$params = array(
							"CDFILIAL"    => $CDFILIAL,
							"NRVENDAREST" => $tableData['NRVENDAREST'],
							"NRCOMANDA"   => $tableData['NRCOMANDA'],
							"VRDESCFID"   => '0'
						);
						$this->entityManager->getConnection()->executeQuery('UPDATE_COMANDAVEN_LIMPA_VRDESCFID', $params);
						$this->entityManager->getConnection()->executeQuery('LIMPA_FIDELIDADE', $params);
					}
					if ($recalcResult['status']){
						$result['status']  = true;
						$result['message'] = $this->waiterMessage->getMessage('457');
						$result['data']    = $tableData;
					} else {
						$result['message'] = $recalcResult['message'];
					}
				} else {
					$result['message'] = $modifyTablePosition['message'];
				}
			} else {
				$comandData = $this->billService->dadosComanda($CDFILIAL, $NRCOMANDA, $NRVENDAREST, $CDLOJA);
                // Se tiver marcado, não altera o cliente/consumidor - mantêm os mesmos e remove o desconto fidelidade para recalcular o preço.

				$this->updateTableData($chave, array($comandData), $CDCLIENTE, $CDCONSUMIDOR);
				// busca itens a partir da NRVENDAREST e NRCOMANDA
				$arrayItens = $this->vendaAPI->buscaItensPedidos($CDFILIAL, $CDLOJA, array($comandData), array());
				$arrayItens = $this->tableService->removeCancelItens($arrayItens);
				// busca preços e executa update
				$recalcResult = $this->tableService->recalcProductPrice($CDFILIAL, $session['CDLOJA'], $arrayItens, array(), $CDCLIENTE, $CDCONSUMIDOR, false);
				if ($recalcResult['status']){
					$result['status']  = true;
					$result['message'] = $this->waiterMessage->getMessage('457');
					$result['data']    = $comandData;
				} else {
					$result['message'] = $recalcResult['message'];
				}
			}
		}

		if ($result['status']){
			$this->entityManager->getConnection()->commit();
		} else {
			$this->entityManager->getConnection()->rollBack();
		}
		return $result;
	}

	public function handleAccountItems($dataset){
		$impPosicao = $dataset['posicao'];
		$NRCOMANDA = $dataset['NRCOMANDA'];
		$NRVENDAREST = $dataset['NRVENDAREST'];
		$session = $this->util->getSessionVars($dataset['chave']);
        $FILIALVIGENCIA = $session['FILIALVIGENCIA'];
		$NRCONFTELA = $session['NRCONFTELA'];
        $DTINIVIGENCIA = new \DateTime($session['DTINIVIGENCIA']);
		if ($dataset['mode'] === 'C') {
			// valida e busca dados da comanda
			$valComanda = $this->billService->dadosComanda($session['CDFILIAL'], $NRCOMANDA, $NRVENDAREST, $session['CDLOJA']);
			$stNrVendaRest = $valComanda['NRVENDAREST'];
			$stNrComanda = $valComanda['NRCOMANDA'];
		}
		else if ($dataset['mode'] === 'M') {
			// valida e busca os dados da mesa
			$valMesa = $this->tableService->dadosMesa($session['CDFILIAL'], $session['CDLOJA'], $NRCOMANDA, $NRVENDAREST);
			$stNrVendaRest = $valMesa['NRVENDAREST'];
			$stNrComanda = $valMesa['NRCOMANDA'];

		    // Retorna todas as mesas de um agrupamento.
		    $r_groupedTables = $this->entityManager->getConnection()->fetchAll("SQL_GET_GROUPED_TABLES", array($valMesa['NRMESA']));
		    // Formata as mesas.
		    $groupedTables = array();
		    foreach ($r_groupedTables as $table){
			    $temp = array(
			  	    'NRCOMANDA' => $table['NRCOMANDA'],
			  	 	'NRVENDAREST' => $table['NRVENDAREST']
			    );
			    array_push($groupedTables, $temp);
		  }
		}

		if (Empty($groupedTables)){
			$stComandaVens = "_".$stNrComanda."_";
		}
		else {
			$stComandaVens = "_";
			foreach ($groupedTables as $mesa){
				$dadosMesa = $this->tableService->dadosMesa($session['CDFILIAL'], $session['CDLOJA'], $mesa['NRCOMANDA'], $mesa['NRVENDAREST']);
				$stComandaVens .= $dadosMesa['NRCOMANDA']."_";
			}
		}

		if ($impPosicao == '') $impPosicao = 'T';

		if(is_array($impPosicao)){
			if (empty($impPosicao)){
				$arrayPosicoes[0] = 'T';
				$impPosicao = "T";
			} else {
				$arrayPosicoes = $impPosicao;
				$impPosicao = implode(",", $arrayPosicoes);
			}
		} else {
			 if ($impPosicao == '')
				$impPosicao = 'T';
				$arrayPosicoes = array($impPosicao);
		}

		$stPosicoes = $impPosicao;
		if($impPosicao != "T"){
			$stPosicoes = "_";
			foreach ($arrayPosicoes as $posicao) {
				$stPosicoes .= $posicao . "_";
			}
		}

		$result = array();
		$params = array(
            'FILIALVIGENCIA' => $FILIALVIGENCIA,
			'NRCONFTELA' => $NRCONFTELA,
            'DTINIVIGENCIA' => $DTINIVIGENCIA,
			'CDFILIAL' => $session['CDFILIAL'],
			'NRCOMANDA' => $stComandaVens,
			'NRLUGARMESA' => $stPosicoes,
			'IMPPOSICAO' => $impPosicao
		);
        $types = array(
            'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
        );
		$r_ItensDetalhados = $this->entityManager->getConnection()->fetchAll("SQL_ITENS_DETALHADOS", $params, $types);
        $r_ItensDetalhados = $this->precoAPI->subgroupDiscountTableInterface($r_ItensDetalhados, $session['CDFILIAL'], $session['CDLOJA']);

		if (empty($r_ItensDetalhados)){
			if ($dataset['mode'] == 'M') return array('funcao' => '0', 'error' => '006'); // Não foi realizado nenhum pedido para esta mesa/comanda.
			else return array('funcao' => '0', 'error' => '430');
		}
		//$produtosCampanha = $this->parametrosAPI->buscaCampanhaCompreGanhe($session['CDFILIAL']);
		$listaItens = array();
		foreach($r_ItensDetalhados as &$item) {
			if ($item['CDPRODUTO'] != $session['CDPRODCOUVER']) {

				if (empty($item['GRUPO'])) {
					$item['GRUPO'] = 'ACRESCIMOS';
				}
				/*$campanha = null;
				foreach ($produtosCampanha as $prod){
					if($item['CDPRODUTO'] == $prod['CDPRODUTO']){
						$campanha = $prod['CDCAMPCOMPGANHE'];
					}
			    }*/

				$listaItens[$item['NRCOMANDA'] . $item['NRPRODCOMVEN']] = array(
					'DSBUTTON'       => $item['DSBUTTON'],
					'NRVENDAREST'    => $item['NRVENDAREST'],
					'nrcomanda'      => $item['NRCOMANDA'],
					'NRPRODCOMVEN'   => $item['NRPRODCOMVEN'],
					'CDPRODPROMOCAO' => $item['CDPRODPROMOCAO'],
					'NRSEQPRODCOM'   => $item['NRSEQPRODCOM'],
					'NRSEQPRODCUP'   => $item['NRSEQPRODCUP'],
					'TXPRODCOMVEN'   => $item['TXPRODCOMVEN'],
                    'IDPESAPROD'     => $item['IDPESAPROD'],
					'GRUPO'          => $item['GRUPO'],
					'codigo'         => $item['CDPRODUTO'],
					'CDPRODUTO'      => $item['CDPRODUTO'],
					'quantidade'     => number_format($item['QTPRODCOMVEN'], 3, ',', ''),
					'preco'          => number_format(floatval(bcsub(str_replace(',','.',strval(((($item['VRPRECCOMVEN'] + $item['VRPRECCLCOMVEN']) * $item['QTPRODCOMVEN']) + $item['VRACRCOMVEN']))), strval($item['VRDESCCOMVEN']), '2')),2,',',''),
					'QTPRODCOMVEN'   => floatval($item['QTPRODCOMVEN']),
					'VRPRECCOMVEN'   => floatval($item['VRPRECCOMVEN'] + $item['VRPRECCLCOMVEN']),
					'VRPRECCLCOMVEN' => floatval($item['VRPRECCLCOMVEN']),
					'desconto'       => number_format($item['VRDESCCOMVEN'], 2, ',', ''),
					'posicao'        => $item['NRLUGARMESA'],
					'composicao'     => null,
					'DTHRINCOMVEN'   => substr($item['HORA'],0,5).' - '.substr($item['DATA'],0,5),
					'IDDIVIDECONTA'  => $item['IDDIVIDECONTA'],
					'NRPRODORIG'     => $item['NRPRODORIG'],
					'NRMESA'         => $item['NRMESA'],
                    'toGoText'       => !empty($item['IDORIGEMVENDA']) ? 'PARA VIAGEM' : '',
                    'NMVENDEDOR'	 => isset($item['NMVENDEDOR']) ? $item['NMVENDEDOR'] : '',
					'PRITOTITEM'	 => floatval($item['VRPRECCOMVEN'] + $item['VRPRECCLCOMVEN'] + $item['VRACRCOMVEN']),
					'IDPESAPROD'     => $item['IDPESAPROD'],
					'VOUCHER'        => null,
					'IDTIPOCOMPPROD' => $item['IDTIPOCOMPPROD']
				);
                if ($item['CDPRODPROMOCAO'] || $item['IDTIPOCOMPPROD'] =='6' ){
					$composicaotemp = $this->buscaComposicao($item['CDFILIAL'], $item['NRVENDAREST'], $item['NRCOMANDA'], $item['NRPRODCOMVEN'], $item['NRSEQPRODCOM']);
					if(!empty($composicaotemp)){
						$listaItens[$item['NRCOMANDA'] . $item['NRPRODCOMVEN']]['composicao'] = $composicaotemp;
					}
				}
			}

		}

		$itens = Array();
		foreach ($listaItens as $item) {
			$item['POS'] = intval($item['posicao']); //Required in order to group items by position.
			$item['posicao'] = 'posição ' . (string)intval($item['posicao']);
			$item['preco'] = '' . $item['preco'];

			array_push($itens, $item);
		};
		$itens['funcao'] = '1';
		return $itens;
	}

	private function buscaComposicao($filial, $nrVendaRest, $nrComanda, $nrProdComVen, $nrSeqProdCom){
		$params = array(
			$filial,
			$nrVendaRest,
			$nrComanda,
			$nrProdComVen
		);
		return $this->entityManager->getConnection()->fetchAll("SQL_ITENS_ITCOMANDAEST", $params);
	}


	public function divideProdutos($dataset){
		$connection = null;

		try {
			$connection = $this->entityManager->getConnection();
	        $connection->beginTransaction();

			$session      = $this->util->getSessionVars($dataset['chave']);
			$CDFILIAL     = $session['CDFILIAL'];
            $FILIALVIGENCIA = $session['FILIALVIGENCIA'];
			$NRCONFTELA   = $session['NRCONFTELA'];
            $DTINIVIGENCIA = new \DateTime($session['DTINIVIGENCIA']);
			$NRVENDAREST  = $dataset["NRVENDAREST"];
			$NRCOMANDA    = $dataset["NRCOMANDA"];
			$NRPRODCOMVENS = $dataset["NRPRODCOMVENS"];
			$NRLUGARMESA  = $dataset["NRLUGARMESA"];

			$NRCOMANDA = $NRCOMANDA[0];
			$NRVENDAREST = $NRVENDAREST[0];

			$numberOfParts = count($NRLUGARMESA);
            $sequenceArray = array(); // Array para guardar NRSEQPRODCOM de promoções.

			foreach ($NRPRODCOMVENS as $NRPRODCOMVEN){
                // Busca o produto.
				$params = array(
                    'FILIALVIGENCIA' => $FILIALVIGENCIA,
					'NRCONFTELA' => $NRCONFTELA,
                    'DTINIVIGENCIA' => $DTINIVIGENCIA,
					'CDFILIAL' => $CDFILIAL,
					'NRCOMANDA' => $NRCOMANDA,
					'NRVENDAREST' => $NRVENDAREST,
					'NRPRODCOMVEN' => $NRPRODCOMVEN
				);
                $types = array(
                    'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
                );
				$itemToSplit = $this->entityManager->getConnection()->fetchAssoc('SQL_ITENS_COMANDA', $params, $types);
                // Insere produto original na ITCMDVENORIG.
				$params = array(
					$CDFILIAL,
					$NRVENDAREST,
					$NRCOMANDA,
					$NRPRODCOMVEN
				);
				$this->entityManager->getConnection()->executeQuery('SQL_INSERE_ITEM_COMANDA_ORIG', $params);
                // Verifica o tipo da promoção. Se tiver itens na ITCOMANDAEST, precisam ser inseridos na ITCMDESTORIG.
                $produtosEST = array();

                //if ($itemToSplit['CDPRODPROMOCAO']){
                    $params = array(
                        'CDFILIAL' => $CDFILIAL,
                        'NRVENDAREST' => $NRVENDAREST,
                        'NRCOMANDA' => $NRCOMANDA,
                        'NRPRODCOMVEN' => $NRPRODCOMVEN
                    );
                    $produtosEST = $this->entityManager->getConnection()->fetchAll("SELECT_ITCOMANDAEST_SPECIFIC", $params);
                    // Insere produto na ITCMDESTORIG.
                    foreach ($produtosEST as $promoItem){
                        $params = array(
                            $promoItem['CDFILIAL'],
                            $promoItem['NRVENDAREST'],
                            $promoItem['NRCOMANDA'],
                            $promoItem['NRPRODCOMVEN'],
                            $promoItem['CDPRODUTO']
                        );
                        $this->entityManager->getConnection()->executeQuery('SQL_INSERE_ITEMEST_ORIG', $params);
                    }
                //}

                $adicionalItem = 0;
                // Divide o produto pelas posições escolhidas.
				for ($pos = 0; $pos < $numberOfParts; $pos++) {
                    // Calcula um novo NRPRODCOMVEN.
					$posicao = $NRLUGARMESA[$pos];
					$this->util->newCode('ITCOMANDAVEN'.$CDFILIAL.$NRCOMANDA);
					$NRPRODCOMVENNOVO = $this->util->getNewCode('ITCOMANDAVEN'.$CDFILIAL.$NRCOMANDA , 6);
                    // Divide o desconto, acréscimo, e quantidade.
					$itemDesc = $this->trunc($itemToSplit['VRDESCCOMVEN'] / $numberOfParts, 2);
					$itemAcr = $this->trunc($itemToSplit['VRACRCOMVEN'] / $numberOfParts, 2);
					$itemQuantity = $this->trunc($itemToSplit['QTPRODCOMVEN'] / $numberOfParts, 3);
                    // Ajusta os valores do último item para compensar.
					if ($pos == $numberOfParts - 1) {
						$multipliedItem = $this->trunc($itemToSplit['VRPRECCOMVEN'] * $itemQuantity, 2);
						$itemCalculatedValue = $multipliedItem * ($numberOfParts - 1);
						$itemQuantity = $this->adjustValueIfNecessary($itemQuantity, $numberOfParts, $itemToSplit['QTPRODCOMVEN'], 3);
						$multipliedItem = $this->trunc($itemToSplit['VRPRECCOMVEN'] * $itemQuantity, 2);
						$itemCalculatedValue += $multipliedItem;

						$itemDesc = $this->adjustValueIfNecessary($itemDesc, $numberOfParts, $itemToSplit['VRDESCCOMVEN'], 2);
						$itemAcr = $this->adjustValueIfNecessary($itemAcr, $numberOfParts, $itemToSplit['VRACRCOMVEN'], 2);

						$itemOriginalValue = $this->trunc($itemToSplit['VRPRECCOMVEN'] * $itemToSplit['QTPRODCOMVEN'], 2);
						if ($itemCalculatedValue != $itemOriginalValue) {
							$adicionalItem = intval(($itemOriginalValue - $itemCalculatedValue) * 100);
						}
					}
                    $NRSEQPRODCOM = null;
                    // Se o produto for promoção, calcula um novo NRSEQPRODCOM.
                    if (!empty($itemToSplit['NRSEQPRODCOM'])){
                        // Verifica se o NRSEQPRODCOM para o conjunto de produtos de uma promoção já foi calculado.
                        if (!empty($sequenceArray[$itemToSplit['NRSEQPRODCOM']][$pos])){
                            $NRSEQPRODCOM = $sequenceArray[$itemToSplit['NRSEQPRODCOM']][$pos];
                        }
                        else {
                            // Cria um novo NRSEQPRODCOM e armazena no array.
                            $params = array(
                                'CDFILIAL' => $CDFILIAL,
                                'NRVENDAREST' => $NRVENDAREST,
                                'NRCOMANDA' => $NRCOMANDA,
                                'NRORG' => $session['NRORG']
                            );
                            $NRSEQPRODCOM = $this->entityManager->getConnection()->fetchAssoc("SQL_GET_NRSEQCOM", $params);
                            $NRSEQPRODCOM = str_pad((string) (intval($NRSEQPRODCOM['NRSEQPRODCOM']) + 1), 3, '0', STR_PAD_LEFT);

                            $sequenceArray[$itemToSplit['NRSEQPRODCOM']][$pos] = $NRSEQPRODCOM;
                        }
                    }
                    // Insere item novo na ITCOMANDAVEN com quantidade, desconto, e acréscimo ajustados.
					$currentItem = array(
						'NRPRODCOMVENNOVO' => $NRPRODCOMVENNOVO,
						'QTPRODCOMVEN'     => $itemQuantity,
						'VRDESCCOMVEN'     => $itemDesc,
						'NRLUGARMESA'      => $posicao,
						'VRACRCOMVEN'      => $itemAcr,
						'NRSEQPRODCOM'     => $NRSEQPRODCOM,
						'NRPRODORIG'       => $NRPRODCOMVEN,
						'CDFILIAL'         => $CDFILIAL,
						'NRVENDAREST'      => $NRVENDAREST,
						'NRCOMANDA'        => $NRCOMANDA,
						'NRPRODCOMVEN'     => $NRPRODCOMVEN
					);
					$this->entityManager->getConnection()->executeQuery('SQL_INSERE_ITEM_POSICAO', $currentItem);
                    // Insere produtos na ITCOMANDAEST caso existam.
                    foreach ($produtosEST as $promoItem){

						// Divide o desconto, acréscimo, e quantidade.
						$itemDesc = $this->trunc($promoItem['VRDESITCOMEST'] / $numberOfParts, 2);
						$itemAcr = $this->trunc($promoItem['VRACRITCOMEST'] / $numberOfParts, 2);
						$itemQuantity = $this->trunc($promoItem['QTPROCOMEST'] / $numberOfParts, 3);

						// Ajusta os valores do último item para compensar.
						//if ($pos == $numberOfParts - 1) {
							$itemQuantity = $this->adjustValueIfNecessary($itemQuantity, $numberOfParts, $promoItem['QTPROCOMEST'], 3);

							$itemDesc = $this->adjustValueIfNecessary($itemDesc, $numberOfParts, $promoItem['VRDESITCOMEST'], 2);
							$itemAcr = $this->adjustValueIfNecessary($itemAcr, $numberOfParts, $promoItem['VRACRITCOMEST'], 2);
						//}

						$itemEST = array(
							$promoItem['CDFILIAL'],
							$NRVENDAREST,
							$NRCOMANDA,
							$NRPRODCOMVENNOVO,
							$promoItem['CDPRODUTO'],
							$itemQuantity,
							$promoItem['VRPRECCOMEST'],
							$itemDesc,
							$promoItem['TXPRODCOMVENEST'],
							$promoItem['NRATRAPRODCOES'],
							$promoItem['DSOBSPEDDIGEST'],
							$itemAcr,
							$promoItem['IDIMPPRGRPROMO']
						);
						$this->entityManager->getConnection()->executeQuery('SQL_INS_ITCOMANDAEST', $itemEST);
					}
				}
                // Apaga produtos associados da ITCOMANDAEST.
                $params = array(
                    $CDFILIAL,
                    $NRVENDAREST,
                    $NRCOMANDA,
                    $NRPRODCOMVEN
                );
                //if (!empty($produtosEST)){
                    $this->entityManager->getConnection()->executeQuery('SQL_DELETA_PROMOEST_ORIGINAL', $params);
                //}
                // Apaga o produto original da ITCOMANDAVEN.
				$this->entityManager->getConnection()->executeQuery('SQL_DELETA_PRODUTO_ORIGINAL', $params);
                // Valida valores.
				$values = $this->entityManager->getConnection()->fetchAll('SQL_VALIDA_VALORES', $params);
				foreach ($values as $value){
                    // Cancela a divisão caso o total fique negativo.
					if (round($value['VRTOTAL'], 2) <= 0){
                        $params = array(
                            'CDFILIAL' => $CDFILIAL,
                            'NRVENDAREST' => $NRVENDAREST,
                            'NRCOMANDA' => $NRCOMANDA,
                            'NRPRODCOMVEN' => $NRPRODCOMVEN
                        );
                        $this->entityManager->getConnection()->executeQuery('SQL_INSERE_ITEM_COMANDAEST', $params);
                        $this->entityManager->getConnection()->executeQuery('SQL_DELETA_CMD_EST_ORIG', $params);
						$params['IDPRODPRODUZ'] = null;
						$this->entityManager->getConnection()->executeQuery('SQL_INSERE_ITEM_COMANDA', $params);
						$this->entityManager->getConnection()->executeQuery('SQL_DELETA_CMD_VEN_ORIG', $params);
					}
				}
			}

			$connection->commit();
			return array('funcao' => '1');

		} catch(\Exception $e){
			Exception::logException($e);
			if ($connection != null) {
            	$connection->rollback();
        	}
			throw new \Exception ($e->getMessage(), 1);
		}
	}

	private function adjustValueIfNecessary($valueToAdjust, $numberOfParts, $totalValue, $handleDecimalValue) {
		$totalCalculatedQuantity = $valueToAdjust * $numberOfParts;
		if ($totalCalculatedQuantity != $totalValue) {
			$valueToAdjust += round($totalValue - $totalCalculatedQuantity, $handleDecimalValue);
		}
		return $valueToAdjust;
	}

	private function trunc($value, $decimals = 2) {
		return floatval(bcdiv(str_replace(',','.',strval($value)), '1', $decimals));
	}

	public function fechaContaMesa($dataset){ //Detaset Params: consumacao, couvert, mesa, pessoas, servico, valorConsumacao
		$connection = null;
		try {
			$session = $this->util->getSessionVars($dataset['chave']);
			/* Open connection and begin transaction. */
			$connection = $this->entityManager->getConnection();
			$connection->beginTransaction();

			$modo = $dataset['modo'];

			if ($modo == 'C') {
				// valida e busca dados da comanda
				$valComanda = $this->billService->dadosComanda($session['CDFILIAL'], $dataset['NRCOMANDA'], $dataset['NRVENDAREST'], $session['CDLOJA']);
				$valComanda['NRMESA'] = str_pad($valComanda['NRMESA'], 4, '0', STR_PAD_LEFT); // ajusta

				$NRMESA = $valComanda['NRMESA'];

				$comandas = array();
				array_push($comandas, $valComanda);

			} else if ($modo == 'M') {
				// valida e busca dados da mesa
				$valMesa = $this->tableService->dadosMesa($session['CDFILIAL'], $session['CDLOJA'], $dataset['NRCOMANDA'], $dataset['NRVENDAREST']);
				$valMesa['NRMESA'] = str_pad($valMesa['NRMESA'], 4, '0', STR_PAD_LEFT);

				$NRMESA = $valMesa['NRMESA'];

				$comandas = $this->billService->buscaComandas($dataset['chave'], $valMesa['NRMESA']);
			}

			if ($dataset['consumacao'] && $session['IDCONSUMAMIN'] === 'S') {
				$this->tableService->controlaConsumacao($dataset['chave'], $dataset['NRVENDAREST'], $dataset['NRCOMANDA'], 'I');
			}

			if ($session['IDCOUVERART'] === 'S'){
				if ($dataset['couvert']) {
					$this->tableService->controlaCouvert($dataset['chave'], $dataset['NRVENDAREST'], $dataset['NRCOMANDA'], 'I', $modo);
					$IDUTILCOUVERT = 'S';
				} else {
					$this->tableService->controlaCouvert($dataset['chave'], $dataset['NRVENDAREST'], $dataset['NRCOMANDA'], 'R', $modo);
					$IDUTILCOUVERT = 'N';
				}
				$this->utilCouvert($session['CDFILIAL'], $dataset['NRCOMANDA'], $dataset['NRVENDAREST'], $IDUTILCOUVERT, $modo);
			}

			$DTHRMESAFECH = new \DateTime();
			foreach ($comandas as $comanda){
				if ($modo == 'M') {
					// altera status da comanda para 2 (conta solicitada)
					$params = array(
						'2',
						$session['CDFILIAL'],
						$comanda['NRCOMANDA'],
						$comanda['NRVENDAREST'],
					);
					$this->entityManager->getConnection()->executeQuery("SQL_ALTERA_STATUS_COMANDA", $params);
					// atualiza horário de fechamento da mesa
					$params = array(
						'DTHRMESAFECH' => $DTHRMESAFECH,
						'CDFILIAL' => $session['CDFILIAL'],
						'NRVENDAREST' => $comanda['NRVENDAREST'],
						'NRMESA' => $comanda['NRMESA']
					);
					$type = array(
						'DTHRMESAFECH' => \Doctrine\DBAL\Types\Type::DATETIME
					);
					$this->entityManager->getConnection()->executeQuery("SQL_ALTERA_HR_FECHAMENTO_MESA", $params, $type);

					// altera status da mesa para S (conta solicitada)
					$params = array(
						'IDSTMESAAUX' => 'S',
						'CDFILIAL'    => $session['CDFILIAL'],
						'CDLOJA'      => $session['CDLOJA'],
						'NRMESA'      => $comanda['NRMESA']
					);
					$this->entityManager->getConnection()->executeQuery('SQL_ALTERA_STATUS_MESA', $params);
				}
				$tempVRCOMISVENDA = $session['VRCOMISVENDA'];
				if($session['IDCOMISVENDA']=="G"){
					$tempVRCOMISVENDA = 0;
					if($session['IDVRINCIDECONTA']=="S"){
						$tempVRCOMISVENDA = $session['VRPERCCOMISVEND'];
					}
				}
				// Executa o cálculo de taxa de servico da mesa/comanda
				$this->alteraTaxaServico($tempVRCOMISVENDA, $session['CDFILIAL'], $comanda['NRVENDAREST'], $comanda['NRCOMANDA'], $dataset['txporcentservico'], $dataset['servico']);
			}

			$parcialData = array(
				'chave'       => $dataset['chave'],
				'mode'        => $modo,
				'NRCOMANDA'   => $dataset['NRCOMANDA'],
				'NRVENDAREST' => $dataset['NRVENDAREST'],
				'funcao'      => $dataset['IMPRIMEPARCIAL'],
				'agrupamento' => array(),
				'posicao'     => null,
                'updateDiscount' => false
			);

			$dadosParcial = $this->dadosParcial($parcialData);

			//Altera VRCOMISVENDE na tabela Comandaven quando taxa de servico e alterada na opcao Alterar Taxa na tela de fechamento de conta.
            $taxadeservicoParcial = str_replace(",", ".", $dadosParcial['servico']);
            $params = array(
                floatval($taxadeservicoParcial),
                $session['CDFILIAL'],
                $dataset['NRVENDAREST'],
                $dataset['NRCOMANDA']
            );
            $this->entityManager->getConnection()->executeQuery("UPDATE_COMISSAO_VENDA", $params);

			if ($modo === 'M') {
				$this->util->logFOS($session['CDFILIAL'], $session['CDCAIXA'], 'FEC_MES', $session['CDOPERADOR'], null, "Waiter - Fechamento de conta", "Fechamento de conta da mesa " . $NRMESA . ".");
			}
			if ($connection !=null) {
				$connection->commit();
			}
			$paramsImpressora = array();
			if(isset($dadosParcial['dadosImpressao']['paramsImpressora'])){
				$paramsImpressora = $dadosParcial['dadosImpressao']['paramsImpressora'];
				$dadosParcial['dadosImpressao'] = '';
			}
			return array(
				'funcao' => '1',
				'dadosImpressao' => $dadosParcial['dadosImpressao'],
				'paramsImpressora' => $paramsImpressora
			);
		} catch (\Exception $e) {
			Exception::logException($e);
			if ($connection !=null)	$connection->rollback();
			throw new \Exception ($e->getMessage(),1);
		}
	}

	private function alteraTaxaServico($VRCOMISVENDA, $CDFILIAL, $NRVENDAREST, $NRCOMANDA, $TAXA, $SERVICO){
		if($SERVICO){
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'NRVENDAREST' => $NRVENDAREST,
				'NRCOMANDA' => $NRCOMANDA
			);

	    	$dadosComissao = $this->entityManager->getConnection()->fetchAssoc("SQL_GET_COMISSAO_VENDA", $params);
	    	$VRCOMISVENDE = $dadosComissao['VRCOMISVENDE'];
	    	if(!isset($TAXA)) {
		    	$TAXA = empty(floatval($dadosComissao["VRCOMISPOR"])) ? $VRCOMISVENDA : floatval($dadosComissao["VRCOMISPOR"]);
		    } else {
		    	$TAXA = floatval($TAXA);
		    }
		} else {
			$TAXA = 0;
			$VRCOMISVENDE = 0;
		}

		$params = array(
			'VRCOMISPOR'   => $TAXA,
			'VRCOMISVENDE' => $VRCOMISVENDE,
			'CDFILIAL'     => $CDFILIAL,
			'NRVENDAREST'  => $NRVENDAREST,
			'NRCOMANDA'    => $NRCOMANDA
		);
		$this->entityManager->getConnection()->executeQuery('UPDATE_PORCENTAGEM_COMISSAO', $params);
	}

	private function checkEmptyAndGetDefaultValue($paramName, $transactionObject, $newValue) {
		$valueToUpdate = $newValue;
		if (!isset($valueToUpdate)) {
			$valueToUpdate = $transactionObject[$paramName];
		}
		return $valueToUpdate;
	}

	public function finalizaPagamento($dataset){
		$connection = null;
        try {
        	$connection->beginTransaction();

            $NRSEQMOVMOB  = $dataset['NRSEQMOVMOB'];
            $NRSEQMOB     = $dataset['NRSEQMOB'];
            $DSBANDEIRA   = $dataset['DSBANDEIRA'];
            $NRADMCODE    = $dataset['NRADMCODE'];
            $IDADMTASK    = $dataset['IDADMTASK'];
            $IDSTMOV      = $dataset['IDSTMOV'];
            $TXMOVUSUARIO = $dataset['TXMOVUSUARIO'];
            $TXMOVJSON    = $dataset['TXMOVJSON'];
            $CDNSUTEFMOB  = $dataset['CDNSUTEFMOB'];
            $TXPRIMVIATEF = $dataset['TXPRIMVIATEF'];
            $TXSEGVIATEF  = $dataset['TXSEGVIATEF'];
            $params = array($NRSEQMOVMOB);
            $transactionObject = $this->entityManager->getConnection()->fetchAssoc('SQL_BUSCA_TRANSACOES', $params);
            if (empty($transactionObject)) {
            	throw new \Exception("Transação não encontrada", 1);
            }

            $params = array(
                "NRSEQMOB" => self::checkEmptyAndGetDefaultValue('NRSEQMOB', $transactionObject, $NRSEQMOB),
                "NRADMCODE" => self::checkEmptyAndGetDefaultValue('NRADMCODE', $transactionObject, $NRADMCODE),
                "IDADMTASK" => self::checkEmptyAndGetDefaultValue('IDADMTASK', $transactionObject, $IDADMTASK),
                "IDSTMOV" => self::checkEmptyAndGetDefaultValue('IDSTMOV', $transactionObject, $IDSTMOV),
                "TXMOVUSUARIO" => self::checkEmptyAndGetDefaultValue('TXMOVUSUARIO', $transactionObject, $TXMOVUSUARIO),
                "TXMOVJSON" => self::checkEmptyAndGetDefaultValue('TXMOVJSON', $transactionObject, $TXMOVJSON),
                "CDNSUTEFMOB" => self::checkEmptyAndGetDefaultValue('CDNSUTEFMOB', $transactionObject, $CDNSUTEFMOB),
                "TXPRIMVIATEF" => self::checkEmptyAndGetDefaultValue('TXPRIMVIATEF', $transactionObject, $TXPRIMVIATEF),
                "TXSEGVIATEF" => self::checkEmptyAndGetDefaultValue('TXSEGVIATEF', $transactionObject, $TXSEGVIATEF),
                "NRSEQMOVMOB" => $NRSEQMOVMOB
            );
            $this->entityManager->getConnection()->executeQuery('SQL_UPDATE_MOVCAIXAMOB', $params);

            // Busca o tipo de recebimento correto, caso exista.
            $CDTIPORECE = $this->entityManager->getConnection()->fetchAssoc('SQL_GET_TIPORECE', array($DSBANDEIRA));
            if (!empty($CDTIPORECE)) {
                $this->entityManager->getConnection()->executeQuery('SQL_UPDATE_MOVCAIXAMOB_CDTIPORECE', array($CDTIPORECE['CDTIPORECE'], $NRSEQMOVMOB));
            }

            $params = array($NRSEQMOVMOB);
            $retorno = $this->entityManager->getConnection()->fetchAll('SQL_BUSCA_TRANSACOES', $params);

            $OLD_DSBANDEIRA = $retorno[0]['DSBANDEIRA'];

            $params = array(
                $DSBANDEIRA,
                $NRSEQMOVMOB
            );
            if ($OLD_DSBANDEIRA == null) {
                $this->entityManager->getConnection()->executeQuery('SQL_UPDATE_DSBANDEIRA', $params);

				$this->util->LogBandeira("Bandeira em MOVCAIXAMOB atualizada de null para {$DSBANDEIRA}");

            } else {

				$this->util->LogBandeira("Bandeira em MOVCAIXAMOB não atualizada. mantida como {$OLD_DSBANDEIRA}");

			}

            $connection->commit();

            return $retorno;
        } catch (\Exception $e){
        	Exception::logException($e);
        	if ($connection != null) $connection->rollback();
            throw new \Exception ($e->getMessage(), 1);
        }
	}

	public function inicializaPagamento($dataset){
		$connection = null;
		try {
			$session = $this->util->getSessionVars($dataset['chave']);

			$cdfilial     = $session['CDFILIAL'];
			$cdcaixa      = $session['CDCAIXA'];
			$CDVENDEDOR   = $dataset['CDVENDEDOR'] == null ? $session['CDVENDEDOR'] : $dataset['CDVENDEDOR'];
			$NRVENDAREST  = $dataset['NRVENDAREST'];
			$NRCOMANDA    = $dataset['NRCOMANDA'];
			$NRMESA       = $dataset['NRMESA'];
			$NRLUGARMESA  = $dataset['NRLUGARMESA'];
			$CDTIPORECE   = $dataset['CDTIPORECE'];
			$IDTIPMOV     = $dataset['IDTIPMOV'];
			$VRMOV        = $dataset['VRMOV'];
			$DSBANDEIRA   = $dataset['DSBANDEIRA'];
			$IDTPTEF      = $dataset['IDTPTEF'];
			$IDATIVO      = 'S';


			$this->util->newCode('MOVCAIXAMOB'.$cdfilial);
			$NRSEQMOVMOB = $this->util->getNewCode('MOVCAIXAMOB'.$cdfilial, 10);

			$NRSEQVENDA = null; // Modificar quando implementar a baixa na mesa ou posição // Update quando atualizar o MOVCAIXA

			$params = array(
				$NRSEQMOVMOB,
				$cdfilial,
				$cdcaixa,
				$CDVENDEDOR,
				$NRVENDAREST,
				$NRCOMANDA,
				$NRSEQVENDA,
				$NRMESA,
				$NRLUGARMESA,
				$CDTIPORECE,
				'E', // $IDTIPMOV
				$VRMOV,
				$DSBANDEIRA,
				$IDTPTEF,
				$IDATIVO
			);

			$this->entityManager->getConnection()->executeQuery("SQL_INSERT_MOVCAIXAMOB", $params);

			$params = array($NRSEQMOVMOB);
			return $this->entityManager->getConnection()->fetchAll("SQL_BUSCA_TRANSACOES", $params);

		} catch(\Exception $e){
			Exception::logException($e);
			if ($connection != null) $connection->rollback();
			throw new \Exception ($e->getMessage(), 1);
		}
	}

	//tambem imprime
	public function dadosParcial($dataset){
		$connection = null;
		try {
			$session = $this->util->getSessionVars($dataset['chave']);
			$modo        = $dataset["mode"];
			$NRCOMANDA   = $dataset['NRCOMANDA'];
			$NRVENDAREST = $dataset['NRVENDAREST'];
			$tipo        = $dataset["funcao"];
			$cdfilial    = $session["CDFILIAL"];
			$cdloja      = $session["CDLOJA"];
			$nrvendarest = $nrcomanda = $nrpessoas = $dscomanda = $nrmesa = 0;
			$impPosicao  = $dataset['posicao'];
			$cdcaixa 	 = $session["CDCAIXA"];
			$bloqueia 	 = !empty($dataset['bloqueia'])? $dataset['bloqueia'] : false;

			if ($modo === "C") {
				$dadosComanda = $this->billService->dadosComanda($cdfilial, $NRCOMANDA, $NRVENDAREST, $cdloja);

				if ($this->validaComanda($dataset['chave'], $NRCOMANDA, $NRVENDAREST) === false) {
					$this->util->geraLog('Erro: '.$this->waiterMessage->getMessage('004').' \nNRCOMANDA: '.$dataset['NRCOMANDA'].
										' NRVENDAREST: '.$dataset['NRVENDAREST'].' Dataset: '.$dataset.' \n\n',
						$session['CDFILIAL'],
						$session['CDCAIXA'],
						new \Datetime()
					);
					return array('funcao' => '0', 'error' => '004');
				}
				$nrvendarest     = $dadosComanda["NRVENDAREST"];
				$nrcomanda       = $dadosComanda["NRCOMANDA"];
				$nrpessoas       = '1';
				$dscomanda       = $dadosComanda["DSCOMANDA"];
				$mesaComanda     = null;
				$nrmesa          = $this->util->zeroFill($dadosComanda["NRMESA"], 4);
				$IDSTMESAAUX     = 'O';
				$NMVENDEDORABERT = "";
                $NRCPFFID4ALL    = $dadosComanda["NRCPFFID4ALL"];
                $VRDESCPROMO4ALL = $dadosComanda["VRDESCPROMO4ALL"];
                $VRDESCCASH4ALL  = $dadosComanda["VRDESCCASH4ALL"];
				$NRCPFFIDUDS     = $dadosComanda["NRCPFFIDUDS"];
                $VRDESCCASHUDS   = floatval($dadosComanda["VRDESCCASHUDS"]);
                $TELCRMBONUS 	 = $dadosComanda["TELCRMBONUS"];
                $VRDESCCRMBONUS  = $dadosComanda["VRDESCCRMBONUS"];
				$NRCPFINTEPROMO  = null;
				$IDPEDINTEPROMO  = null;
				$DSPROMOCOM      = null;
			} else if ($modo === "M") {
				$dadosMesa = $this->tableService->dadosMesa($cdfilial, $cdloja, $NRCOMANDA, $NRVENDAREST);
				$nrvendarest	   = $dadosMesa["NRVENDAREST"];
				$nrcomanda		   = $dadosMesa["NRCOMANDA"];
				$nrpessoas		   = $dadosMesa["NRPESMESAVEN"];
				$dscomanda		   = "";
				$mesaComanda	   = $dadosMesa['NRMESA'];
				$nrmesa			   = $mesaComanda;
				$IDSTMESAAUX	   = $dadosMesa['IDSTMESAAUX'];
				$NMVENDEDORABERT   = $dadosMesa["NMVENDEDORABERT"];
                $NRCPFFID4ALL      = $dadosMesa["NRCPFFID4ALL"];
                $VRDESCPROMO4ALL   = $dadosMesa["VRDESCPROMO4ALL"];
                $VRDESCCASH4ALL    = $dadosMesa["VRDESCCASH4ALL"];
				$NRCPFFIDUDS       = $dadosMesa["NRCPFFIDUDS"];
                $VRDESCCASHUDS     = floatval($dadosMesa["VRDESCCASHUDS"]);
                $TELCRMBONUS 	   = $dadosMesa["TELCRMBONUS"];
                $VRDESCCRMBONUS    = $dadosMesa["VRDESCCRMBONUS"];
				$NRCPFINTEPROMO    = $dadosMesa["NRCPFINTEPROMO"];
				$IDPEDINTEPROMO    = $dadosMesa["IDPEDINTEPROMO"];
				$DSPROMOCOM        = $dadosMesa["DSPROMOCOM"];
				$params = array($cdfilial, $cdloja, $mesaComanda);
				$r_ambiente = $this->entityManager->getConnection()->fetchAssoc("SQL_AMBIENTE", $params);
				if ($r_ambiente) {
					$ambiente = $r_ambiente["NMSALA"];
				}
			}

			// Retorna todas as mesas de um agrupamento, caso a mesa esteja agrupada.
			$r_groupedTables = $this->entityManager->getConnection()->fetchAll("SQL_GET_GROUPED_TABLES", array($mesaComanda));
			if (empty($r_groupedTables)) {
				// Mesa não está agrupada.
				$stComandaVens = "_".$nrcomanda."_";
				$stVendaRests  = "_".$nrvendarest."_";
			}
			else {
				// Mesa está agrupada.
				$stComandaVens = "_";
				$stVendaRests  = "_";
				//$nrpessoas = 0;
				//nao soma a quantidade porque dados mesa ja retorna a quantidade das mesas agrupadas
				$nrpessoas = $dadosMesa['NRPESMESAVEN'];
				foreach ($r_groupedTables as $table){
					$dadosMesa = $this->tableService->dadosMesa($session['CDFILIAL'], $session['CDLOJA'], $table['NRCOMANDA'], $table['NRVENDAREST']);
					$stComandaVens .= $dadosMesa['NRCOMANDA']."_";
					$stVendaRests  .= $dadosMesa['NRVENDAREST']."_";
					//$nrpessoas += $dadosMesa['NRPESMESAVEN'];
				}
			}
			//Se não for uma posição específica, muda para pegar todas.
			if (is_array($impPosicao)){
				if (empty($impPosicao)){
					$arrayPosicoes[0] = 'T';
					$impPosicao = "T";
				} else {
					$arrayPosicoes = $impPosicao;
					$impPosicao = implode(",", $arrayPosicoes);
				}
			} else {
				if ($impPosicao == '') {
					$impPosicao = 'T';
				}
				$arrayPosicoes = array($impPosicao);
			}

			$stPosicoes = $impPosicao;
			if ($impPosicao != "T") {
				$stPosicoes = "_";
				foreach ($arrayPosicoes as $posicao) {
					$stPosicoes .= $posicao . "_";
				}
			}

			$params = array(
				$cdfilial,
				$stComandaVens,
				$stPosicoes,
				$impPosicao,
				$cdfilial,
				$stComandaVens,
				$stPosicoes,
				$impPosicao
			);
			$produtosParcial = $this->entityManager->getConnection()->fetchAll("SQL_PRODUTOS_PARCIAL", $params);
            $produtosParcial = $this->precoAPI->subgroupDiscountTableInterface($produtosParcial, $session['CDFILIAL'], $session['CDLOJA']);

			$params = array(
				$cdfilial,
				$stVendaRests,
				$stComandaVens,
				$stPosicoes,
				$impPosicao
			);
			$numeroProdutos = $this->entityManager->getConnection()->fetchAssoc("SQL_PRODUTOS_PARCIAL_NUMERO_PRODUTOS", $params);
			$numeroProdutos = $numeroProdutos['NRITENS'];

			if (!is_array($impPosicao)){
				if ($impPosicao == 'T') {
					$impPosicao = '';
				}
			}

			$cdprodcouver = $session["CDPRODCOUVER"];
			$cdprodconsum = $session["CDPRODCONSUM"];

			$totalProdutosTaxa = 0;
			$totalProdutosTaxaDesc = 0;
			$totalSemDesconto = 0;
			$totalDesconto = 0;
            $totalSubsidy = 0;
			self::calculaValoresParcial($cdfilial, $produtosParcial, $cdprodcouver, $cdprodconsum, $totalProdutosTaxa, $totalProdutosTaxaDesc, $totalSemDesconto, $totalDesconto, $totalSubsidy);
			$totalSemDesconto = round($totalSemDesconto, 2);
			$totalProdutosTaxa = round(round($totalProdutosTaxa, 2) - $totalProdutosTaxaDesc, 2);
			$total = round($totalSemDesconto - $totalDesconto, 2);

			$fidelityValue = 0;
            $cashbackUsed4ALL = 0;
			$fidelityDiscount = 0;
			$VRDESCPROMO = $VRDESCPROMO4ALL;
			$VRDESCCASH = $VRDESCCASH4ALL;
			if(!empty($TELCRMBONUS)){
				$VRDESCCASH = $VRDESCCRMBONUS;
			}
			// calcula valores do Crédito Fidelidade
			self::adjustDiscountFidelity($cdfilial, $nrvendarest, $produtosParcial, $cdprodcouver, $cdprodconsum, $totalSemDesconto, $totalDesconto, $numeroProdutos, $arrayPosicoes, $VRDESCPROMO, $VRDESCCASH, $fidelityValue, $cashbackUsed4ALL, $fidelityDiscount, $totalProdutosTaxa, $totalSubsidy, $dataset['updateDiscount']);

			/* Open connection and begin transaction. */
			$connection = $this->entityManager->getConnection();
			$connection->beginTransaction();

			// calcula consumação e couvert com array indexado pela posição com o valor referente
			//Parametro 'C' significa Consulta
			$consumacao = 0;
			if ($session['IDCONSUMAMIN'] === "S"){
				$valoresConsumacao = $this->tableService->controlaConsumacao($dataset['chave'], $nrvendarest, $nrcomanda, 'C');
				if ($arrayPosicoes[0] === 'T') {
					$consumacao += array_sum($valoresConsumacao);
				} else {
					foreach($arrayPosicoes as $posicao) {
						$consumacao += $valoresConsumacao[$posicao];
					}
				}
			}
			$couvert = 0;
			if ($session['IDCOUVERART'] === 'S') {
				$valoresCouvert = $this->tableService->controlaCouvert($dataset['chave'], $nrvendarest, $nrcomanda, 'C', $modo);

				if (!empty($valoresCouvert)){
					if ($arrayPosicoes[0] === 'T') {
						$couvert += array_sum($valoresCouvert);
					} else {
						foreach($arrayPosicoes as $posicao) {
							$couvert += $valoresCouvert[$posicao];
						}
					}
				}
			}
			$total += $consumacao + $couvert;

			// calcula descontos e acréscimos para a comanda
			if ($modo === "C") {
				$dadosComanda = $this->billService->dadosComanda($cdfilial, $nrcomanda, $nrvendarest, $cdloja);
				if ($dadosComanda["VRDESCOMANDA"] > 0) {
					$total -= $dadosComanda["VRDESCOMANDA"];
				}
				$total += $dadosComanda["VRACRCOMANDA"];
			}
 
			// calcula taxa de serviço
			$taxaDeServico = self::taxaDeServico($dataset['chave'], $totalProdutosTaxa, $cdfilial, $nrvendarest, $nrcomanda, $IDSTMESAAUX, $produtosParcial, $VRDESCCASHUDS);
			$total += $taxaDeServico['calculada'];

			// calcula tempo de permanência
			$tempoPermanencia = self::calculaTempoPermanencia($cdfilial, $nrvendarest, $nrcomanda);
			$horas            = $tempoPermanencia["horas"];
			$minutos          = $tempoPermanencia["minutos"];
			$segundos         = $tempoPermanencia["segundos"];

            if ($dataset['updateDiscount']){
                $total = round($total - $fidelityValue, 2);
            }
			// calcula total por pessoa
			$totalPorPessoa = floatval($total) / floatval($nrpessoas);
			if ($connection !=null) {
				$connection->commit();
				$connection = null;
			}

			// os valores já pagos (vendas) foram comentados porque o sistema esta subtraindo esse valor de posições que ainda estão abertas, deixando o valor total da venda errado. Somente so valores da MOVCAIXAMOB vão ser carregados
			$valorPago = 0;//self::valorPago($dataset, $session, $NRVENDAREST, $impPosicao, $nrcomanda);

			$valorAdiantado=0;
			$params = array(
				'CDFILIAL' => $cdfilial,
				'CDCAIXA' => $cdcaixa,
				'NRVENDAREST' => $NRVENDAREST,
				'NRCOMANDA' => $NRCOMANDA
			);
			$advancedPayments = $this->entityManager->getConnection()->fetchAll("GET_ADVANCED_PAYMENTS", $params);
			if(!empty($advancedPayments)) {
				foreach($advancedPayments as $actualAdvancedPayment) {
					$valorPago = $valorPago + floatval($actualAdvancedPayment['VRMOV']);
					$valorAdiantado = $valorAdiantado + floatval($actualAdvancedPayment['VRMOV']);
					$total = floatval($total) - floatval($actualAdvancedPayment['VRMOV']);
				}
			}
			if(empty($impPosicao)){
				$impPosicao = 'T';
			}
			if($impPosicao=='T'){
				if(intval($nrpessoas)>0){
					$impPosicao = Array('T');
					for ($i=0; $i < $nrpessoas; $i++) {
						array_push($impPosicao,str_pad($i+1, 2, "0", STR_PAD_LEFT));
					}
				}
			}

			if(is_array($impPosicao)){
				if(intval($nrpessoas)>0){
					if(intval($nrpessoas)>=count($impPosicao)){
						array_push($impPosicao,'T');
					}
				}
				$impPosicao = join(",",$impPosicao);
			} else {
				//if(intval($nrpessoas)==1){
					$impPosicao = explode(',', $impPosicao);
					for ($i=0; $i < $nrpessoas; $i++) {
						if(!in_array(str_pad($i+1, 2, "0", STR_PAD_LEFT),$impPosicao)){
							array_push($impPosicao,str_pad($i+1, 2, "0", STR_PAD_LEFT));
						}
					}
					if(intval($nrpessoas)>=count($impPosicao)){
						array_push($impPosicao,'T');
					}
					$impPosicao = join(",",$impPosicao);
				//}
			}
			$params = array(
				'NRVENDAREST' => $NRVENDAREST,
				'NRCOMANDA'   => $NRCOMANDA,
				'NRLUGARMESA' => array_filter(explode(',', $impPosicao))
			);
			$types = array(
				'NRLUGARMESA' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
			);
			$arrMOVCAIXAMOB = $this->entityManager->getConnection()->fetchAll("SQL_BUSCA_MOVCAIXAMOB_POSICAO", $params,$types);

			if(!empty($arrMOVCAIXAMOB)) {
				$anterior = null;
				foreach($arrMOVCAIXAMOB as $MOVCAIXAMOB) {
					if($MOVCAIXAMOB['IDSTATUSMOV']!='P'){
						if($anterior){
							if(
								$anterior['VRMOV']==$MOVCAIXAMOB['VRMOV'] &&
								$anterior['DTHRINCMOV']==$MOVCAIXAMOB['DTHRINCMOV'] &&
								$anterior['CDTIPORECE']==$MOVCAIXAMOB['CDTIPORECE'] &&
								$anterior['CDNSUTEFMOB']==$MOVCAIXAMOB['CDNSUTEFMOB'] &&
								$anterior['NRLUGARMESA']!=$MOVCAIXAMOB['NRLUGARMESA']
							){
								continue;
							}
						}
						$valorPago = $valorPago + floatval($MOVCAIXAMOB['VRMOV']);
						$valorAdiantado = $valorAdiantado+ floatval($MOVCAIXAMOB['VRMOV']);
						$total = round(floatval($total) - floatval($MOVCAIXAMOB['VRMOV']), 2);
						$anterior = $MOVCAIXAMOB;
					} else if($MOVCAIXAMOB['IDSTATUSMOV'] == 'P') {
						$valorPago = $valorPago + floatval($MOVCAIXAMOB['VRMOV']);
						$valorAdiantado = $valorAdiantado+ floatval($MOVCAIXAMOB['VRMOV']);
						$total = round(floatval($total) - floatval($MOVCAIXAMOB['VRMOV']), 2);
						$anterior = $MOVCAIXAMOB;
					}
				}
			}

			$dadosImpressao = '';
			// monta arquivo de impressão
			if ($tipo === "I"){
				$this->impressaoService->imprimeParcial($produtosParcial ,$dataset['chave'], $cdfilial, $stVendaRests,
				 $stComandaVens, $cdprodcouver, $modo, $nrvendarest, $nrcomanda, $cdloja, $nrmesa, $impPosicao, $nrpessoas,
				 $totalSemDesconto, $totalDesconto, $taxaDeServico['calculada'], $total, $totalPorPessoa, $horas, $minutos,
				 $segundos, $dscomanda, $dadosImpressao, $fidelityValue, $couvert, $valorPago);
			}

			if ($modo === 'C'){
				//Altera o status da comanda para 7 caso IDBLOQCOMPARC da tabela LOJA seja 'S'
				self::alteraStatusComanda($cdfilial, $nrcomanda, $nrvendarest, $cdcaixa, $bloqueia);
			}

            $realSubsidy = 0;
            foreach ($produtosParcial as $produto){
                $VRPRECCOMVEN = floatval($produto['VRPRECCOMVEN']);
                $VRPRECCLCOMVEN = floatval($produto['VRPRECCLCOMVEN']);
                $DISCOUNT = floatval(bcdiv(str_replace(',','.',strval($produto['VRDESCCOMVEN'])), str_replace(',','.',strval($produto['QTPRODCOMVEN'])), '2'));

                $productSubsidy = self::calculateSubsidy($VRPRECCOMVEN, $VRPRECCLCOMVEN, $DISCOUNT);
                $realSubsidy += floatval(bcmul(str_replace(',','.',strval($produto['QTPRODCOMVEN'])), str_replace(',','.',strval($productSubsidy)), '2'));
			}

            if ($dataset['updateDiscount']){
                $totalDesconto = round($totalDesconto + $fidelityValue, 2);
            }

            if ($VRDESCCASHUDS > 0){
				$totalDesconto = round($totalDesconto + $VRDESCCASHUDS, 2);
				$total = $total - $VRDESCCASHUDS;
			}

            if (floatval(bcmul($total, 1, 2)) == 0){
                $total = round($total, 2);
            }

			return array(
				"pessoas"          		=> $nrpessoas,
				"posicao"          		=> $dataset['posicao'],
				"permanencia"      		=> $horas . $minutos,
				"produtos"         		=> $this->util->formataPreco($totalSemDesconto),
				"desconto"         		=> $this->util->formataPreco($totalDesconto),
				"lblDesconto"	   		=> $this->util->formataPreco($totalDesconto),
				"servico"          		=> $this->util->formataPreco($taxaDeServico['calculada']),
				"couvert"          		=> $this->util->formataPreco($couvert),
				"consumacao"       		=> $this->util->formataPreco($consumacao),
				"total"            		=> $this->util->formataPreco($total),
				"valorPago"        		=> $this->util->formataPreco($valorPago),
				"valorAdiantado"     	=> $this->util->formataPreco($valorAdiantado),
                "totalSubsidy"     		=> $totalSubsidy,
                "realSubsidy"      		=> $realSubsidy,
				"funcao"           		=> '1',
				"numeroProdutos"   		=> $numeroProdutos,
				"dadosImpressao"   		=> $dadosImpressao,
				"fidelityValue"    		=> $fidelityValue,
				"fidelityDiscount"      => $fidelityDiscount,
                "cashbackUsed4ALL"      => $cashbackUsed4ALL,
				"vlrprodcobtaxa"   		=> $totalProdutosTaxa,
				"NMVENDEDORABERT"		=> isset($NMVENDEDORABERT) ? $NMVENDEDORABERT : "",
				"taxaOriginal"          => $taxaDeServico['original'],
                "NRCPFFID4ALL"          => $NRCPFFID4ALL,
				"NRCPFFIDUDS"           => $NRCPFFIDUDS,
				"NRCPFINTEPROMO"        => $NRCPFINTEPROMO,
				"IDPEDINTEPROMO"        => $IDPEDINTEPROMO,
				"NRVENDAREST"           => $nrvendarest,
				"NRCOMANDA" 			=> $nrcomanda,
				"DSPROMOCOM" 	     	=> $DSPROMOCOM,
				"VRDESCCASHUDS" 		=> $VRDESCCASHUDS
			);
		} catch(\Exception $e) {
			Exception::logException($e);
			if ($connection !=null) $connection->rollback();
			throw new \Exception ($e->getMessage(),1);
		}
	}

	private function validaComanda($chave, $NRCOMANDA, $NRVENDAREST){
		$session = $this->util->getSessionVars($chave);
		$params = array(
			$session["CDFILIAL"],
			$NRCOMANDA,
			$NRVENDAREST
		);
		return (bool)$this->entityManager->getConnection()->fetchAssoc("SQL_VALIDA_COMANDA", $params);
	}

	private function calculaValoresParcial($cdfilial, $produtosParcial, $cdprodcouver, $cdprodconsum, &$totalProdutosTaxa, &$totalProdutosTaxaDesc, &$totalSemDesconto, &$totalDesconto, &$totalSubsidy){

        $params = array(
            'CDFILIAL' => $cdfilial
        );
        $IDCONDESCTXSERV = $this->entityManager->getConnection()->fetchAssoc("SQL_FILIAL_DETAILS", $params)['IDCONDESCTXSERV'];

		foreach ($produtosParcial as $item){
			if ($item['CDPRODUTO'] != $cdprodcouver && $item['CDPRODUTO'] != $cdprodconsum) {
				$vrProd = floatval(bcmul(str_replace(',','.',strval($item['QTPRODCOMVEN'])), str_replace(',','.',strval($item['VRPRECCOMVEN'] + $item['VRPRECCLCOMVEN'])), 2)) + $item['VRACRCOMVEN'];
				if ($item['IDCOBTXSERV'] === 'S') {
                    $totalProdutosTaxa += $vrProd;
                    if ($IDCONDESCTXSERV === 'S' || $item['IDDESCMANUAL'] === 'N') {
                        $totalProdutosTaxaDesc += $item['VRDESCCOMVEN'];
                    }
				}
				$totalSemDesconto += $vrProd;
				$totalDesconto += $item['VRDESCCOMVEN'];
                $totalSubsidy += floatval(bcmul(str_replace(',','.',strval($item['QTPRODCOMVEN'])), str_replace(',','.',strval($item['VRPRECCLCOMVEN'])), 2));
			}
		}
	}

	private function valorPago($dataset, $session, $nrvendarest, $impPosicao, $nrcomanda){
		if ($impPosicao === "") {
			$transacoes = $this->transactionsService->buscaTransacoesMesa($session, $nrvendarest, $nrcomanda);
		} else {
			// apenas caixa com IDCOLETOR === 'C' controla o valor pago por posição
			$transacoes = $this->transactionsService->buscaTransacoesPosicao($session, $nrvendarest, $nrcomanda, $impPosicao);
		}

		// operação com os valores recebidos menos troco dado
		$valorPago = $transacoes['VALORPAGO'] - $transacoes['VALORRETIRADA'];

		return round($valorPago, 2);
	}

	private function taxaDeServico($chave, $totalProdutos, $cdfilial, $nrvendarest, $nrcomanda, $IDSTMESAAUX, $produtosParcial, $VRDESCCASHUDS){
		$session = $this->util->getSessionVars($chave);
		$idcomisvenda = $session["IDCOMISVENDA"];
		$VRCOMISVENDA = $session["VRCOMISVENDA"];
		$taxaDeServico = array();
		// Original - Taxa de serviço com o valor inalterado.
		// Calculada - Após alterações manuais da taxa de serviço pelo usuário.
		$taxaDeServico['calculada'] = $taxaDeServico['original'] = 0;

		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda
		);
    	$dadosComissao = $this->entityManager->getConnection()->fetchAssoc("SQL_GET_COMISSAO_VENDA", $params);

		$totalProdutos = $totalProdutos - $VRDESCCASHUDS;
		if($totalProdutos<=0){
			$taxaDeServico['original'] = 0;
			$taxaDeServico['calculada'] = 0;
			return $taxaDeServico;
		}
		
        if ($idcomisvenda == "S") {
			// verifica se a taxa de servico foi alterada
			if ($dadosComissao['VRCOMISPOR'] != $session["VRCOMISVENDA"] && !empty($dadosComissao['VRCOMISPOR'])) {
				$porcentTaxa = $dadosComissao['VRCOMISPOR'];
			} else {
				$porcentTaxa = floatval($session["VRCOMISVENDA"]);
			}

            $taxaCalculada = $this->util->truncate($totalProdutos * $porcentTaxa/100, 2);
            $taxaDeServico['original'] = $this->util->truncate($totalProdutos * $porcentTaxa/100, 2);
        }
        else if ($idcomisvenda == "G"){
            $taxaCalculada = 0;	
			foreach ($produtosParcial as $item){	
				// verifica se a taxa de servico foi alterada
				if ($dadosComissao['VRCOMISPOR'] != $item['VRPERCCOMISVEND'] && !empty($dadosComissao['VRCOMISPOR'])) {
					$porcentTaxa = $dadosComissao['VRCOMISPOR'];
				} else {
					$porcentTaxa = floatval($item['VRPERCCOMISVEND']);
				}
				
                if ($item['CDPRODUTO'] != $session["CDPRODCOUVER"] && $item['CDPRODUTO'] != $session["CDPRODCONSUM"]){
                    $vrProd = round(floatval(bcmul(str_replace(',','.',strval($item['QTPRODCOMVEN'])), str_replace(',','.',strval(($item['VRPRECCOMVEN'] + $item['VRPRECCLCOMVEN']))), 2)) - $item['VRDESCCOMVEN'] + $item['VRACRCOMVEN'], 2);
                    
					if ($item['IDCOBTXSERV'] == 'S' && $item['IDVRINCIDECONTA'] == "S"){
                        $taxaCalculada += $this->util->truncate($vrProd * $porcentTaxa/100, 2);
                    }

                }
            }

            $taxaDeServico['original'] = $taxaCalculada;
        }

        if ($idcomisvenda != "N"){
            // caso existir, retorna o valor calculado se for diferente de zero; se não, salva e retorna
            if ($IDSTMESAAUX !== 'O') {
				if (floatval($dadosComissao['VRCOMISVENDE']) != 0 || $dadosComissao['VRCOMISPOR'] > 0) {
					$taxaDeServico['calculada'] = $taxaCalculada;
				}
			} else {
				if ($taxaCalculada > 0) {
					$params = array(
						$taxaCalculada,
						$cdfilial,
						$nrvendarest,
						$nrcomanda
					);
					$this->entityManager->getConnection()->executeQuery("UPDATE_COMISSAO_VENDA", $params);
				}
				$taxaDeServico['calculada'] = $taxaCalculada;
			}
        }
		return $taxaDeServico;
	}

	private function calculaTempoPermanencia($cdfilial, $nrvendarest, $nrcomanda){
		$params = array($cdfilial, $nrvendarest, $nrcomanda);
		$tempoPermanencia = $this->entityManager->getConnection()->fetchAssoc("SQL_TEMPO_PERMANENCIA", $params);
		$now     = $this->date->getDataAtual();
		// @todo validar o formato de saida do campo "$tempoPermanencia["DATA"]".
		// Assume-se aqui que seu formato é d/m/Y H:i:s
		$entrada           = $this->date->getDataDeString($tempoPermanencia["DATA"], "Y-m-d H:i:s.u");
		$intervalo         = $entrada->diff($now);
		$intervaloHoras    = str_pad((int)$intervalo->format('%R%h'), 2, '0', STR_PAD_LEFT);
		$intervaloMinutos  = str_pad((int)$intervalo->format('%R%i'), 2, '0', STR_PAD_LEFT);
		$intervaloSegundos = str_pad((int)$intervalo->format('%R%s'), 2, '0', STR_PAD_LEFT);

		return array (
			"horas"    => $intervaloHoras,
			"minutos"  => $intervaloMinutos,
			"segundos" => $intervaloSegundos
		);
	}
    
    public function listNotas($cdfilial, $nrNotaFiscal, $cdCaixaLogado){
		$params = array('CDFILIAL' => $cdfilial, 'NRNOTAFISCALCE' => $nrNotaFiscal, 'CDCAIXALOGADO' => $cdCaixaLogado);
        $notasEncontrada = $this->entityManager->getConnection()->fetchAll("SQL_GET_VENDA_BY_NRNOTAFISCALCE_V2",$params);
		foreach ($notasEncontrada as &$nota) {
			$nota['VRTOTVENDA'] = number_format($nota['VRTOTVENDA'], 2, ',', '.');
			$nota['DTEMISSAONFCE'] = date_format(date_create($nota['DTEMISSAONFCE']), 'd/m/Y H:i:s');
		}
        return $notasEncontrada;
    }
    
	public function saleCancel($CODIGOCUPOM, $chave, $CDSUPERVISOR, $DSOBSCANCUP, $CDCAIXA_CUPOM ){
        $session = $this->util->getSessionVars($chave);
        $CDFILIAL = $session['CDFILIAL'];
        $CDCAIXA = $session['CDCAIXA'];
        $connection = $this->entityManager->getConnection();
        $resCancelBalance = true;

		$venda = $this->getVendabyCupomCaixaFilial($CDFILIAL, $CODIGOCUPOM, $CDCAIXA_CUPOM);

		$result = array(
			'error' => true,
			'message' => '',
			'data' => array(
				'dadosImpressao' => array(),
				'dataTEF' => array()
			)
		);

        try {
    		if (!empty($venda)){
				if($CDCAIXA != $CDCAIXA_CUPOM){
					$params = array($CDFILIAL, $CDCAIXA);
					$arrDadosCaixa = $this->entityManager->getConnection()->fetchAssoc("SQL_DADOS_CAIXA", $params);
					$params = array($CDFILIAL, $CDCAIXA_CUPOM);
					$arrDadosCaixaCupom = $this->entityManager->getConnection()->fetchAssoc("SQL_DADOS_CAIXA", $params);

					if($arrDadosCaixa['IDTPEMISSAOFOS']=='SAT' && $arrDadosCaixaCupom['IDTPEMISSAOFOS']=='SAT'){
						$stPerifericos = empty($arrDadosCaixa['DSSATHOST'])?$arrDadosCaixa['CDURLWSSAT']:$arrDadosCaixa['DSSATHOST'];
						$stPerifericosCaixaCupom = empty($arrDadosCaixaCupom['DSSATHOST'])?$arrDadosCaixaCupom['CDURLWSSAT']:$arrDadosCaixaCupom['DSSATHOST'];
						if(empty($stPerifericos) || empty($stPerifericosCaixaCupom)){
							throw new \Exception ('Para o cancelamento de vendas SAT o caixa deve utilizar o mesmo SAT compartilhado.', 1);
						}else {

							$arrProcura = array("http://","/api","/service","/index.php","/apiFos");
							$stPerifericos = str_replace($arrProcura, "", $stPerifericos);
							$stPerifericosCaixaCupom = str_replace($arrProcura, "", $stPerifericosCaixaCupom);
							if( strpos($stPerifericos, ":") >= 0 ){
								$stPerifericos = substr($stPerifericos, 0, strpos($stPerifericos, ":"));
							}
							if( strpos($stPerifericosCaixaCupom, ":") >= 0 ){
								$stPerifericosCaixaCupom = substr($stPerifericosCaixaCupom, 0, strpos($stPerifericosCaixaCupom, ":"));
							}
							if($stPerifericos != $stPerifericosCaixaCupom){
								throw new \Exception ('Para o cancelamento de vendas SAT o caixa deve utilizar o mesmo endereço do SAT compartilhado.', 1);
							}
						}
					} else if($arrDadosCaixa['IDTPEMISSAOFOS']!=$arrDadosCaixaCupom['IDTPEMISSAOFOS']){
						throw new \Exception ('Não é possivel cancelar a venda de caixas que possuem tipos de emissão diferentes.', 1);
					}
				}

				if(!empty($session['CDURLFIDONLINE']) && $venda['CDCONSUMIDOR'] != null) {
					$resCancelBalance = $this->fidelidadeOnline->curlCancelBalance($session['CDURLFIDONLINE'], $CDFILIAL, $CDCAIXA_CUPOM, $venda['NRSEQVENDA'], $venda['CDCLIENTE'], $venda['CDCONSUMIDOR'], $session['DTABERCAIX']);
				}

    			$result['data']['dataTEF'] = self::getTransactions($venda['NRSEQVENDA'], $CDFILIAL, $CDCAIXA_CUPOM, $session['NRORG']);
                $connection->beginTransaction();
    			$resultCancel = $this->vendaAPI->cancelaCupom($CDFILIAL, $CDCAIXA, $session['NRORG'], $venda['NRSEQVENDA'], $session['CDOPERADOR'], $CDSUPERVISOR, $DSOBSCANCUP, $CDCAIXA_CUPOM);
    			if (!$resultCancel['error']) {
                    $connection->commit();
                    $result['error'] = false;
                    $result['message'] = 'Venda cancelada com sucesso.';
    				// formata mensagem de acordo com os erros do fluxo de cancelamento
    				if (isset($resultCancel['mensagemNfce']) && $resCancelBalance !== false){
    					$result['message'] .= ' <br><br> ' . $resultCancel['mensagemNfce'];
    				}
					else if (isset($resultCancel['mensagemNfce']) && $resCancelBalance === false){
						$result['message'] = 'Venda cancelada com sucesso, mas com erro ao cancelar saldo fidelidade.';
    					$result['message'] .= ' <br><br> ' . $resultCancel['mensagemNfce'];
    				}
                    else if (isset($resultCancel['mensagemImpressao'])){
						$result['message'] .= ' <br><br> ' . $resultCancel['mensagemImpressao'];
    				}
    				$result['data']['dadosImpressao'] = $resultCancel['dadosImpressao'];
    			} else {
                    if ($connection != null){
                        $connection->rollback();
                    }
    				$result = $resultCancel;
    			}
    		}
            else {
    			$result['message'] = 'Venda para este código de cupom não encontrada ou não relacionada a abertura deste caixa.';
    		}

    		return $result;
        } catch (\Exception $e){
            if ($connection != null){
				if ($connection->isTransactionActive()) {
                	$connection->rollback();
            	}
            }
            $result['message'] = $e->getMessage();
            return $result;
        }
	}

	public function getVendabyCupomCaixaFilial($CDFILIAL, $NRNOTAFISCALCE, $CDCAIXA){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRNOTAFISCALCE' => $NRNOTAFISCALCE
        );
        return $this->entityManager->getConnection()->fetchAssoc('SQL_GET_VENDA_BY_NRNOTAFISCALCE_V3', $params);
	}
    
    public function getVendabyCupomFilial($CDFILIAL, $NRNOTAFISCALCE){
        $params = array(
        'CDFILIAL' => $CDFILIAL,
        'NRNOTAFISCALCE' => $NRNOTAFISCALCE
        );
        return $this->entityManager->getConnection()->fetchAssoc('SQL_GET_VENDA_BY_NRNOTAFISCALCE_V2', $params);
    }
    

	public function getTransactions($NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG){
		$params = array(
			'NRSEQVENDA' => $NRSEQVENDA,
			'CDFILIAL' 	 => $CDFILIAL,
			'CDCAIXA' 	 => $CDCAIXA,
			'NRORG' 	 => $NRORG
		);

		$recebimentos = $this->entityManager->getConnection()->fetchAll('SQL_GET_MOVCAIXA_BY_NRSEQVENDA', $params);

		return array_filter($recebimentos, function($recebimento){
			return !empty($recebimento['NRCONTROLTEF']);
		});
	}

	public function changeProductDiscount($NRVENDAREST, $NRCOMANDA, $VRDESCONTO, $TIPODESCONTO, $NRPRODCOMVEN, $CDSUPERVISOR, $motivoDesconto, $CDGRPOCORDESC){
		$result = array(
			'error' => true,
			'message' => ''
		);
		$connection = null;

		try {
			$connection = $this->entityManager->getConnection();
			$connection->beginTransaction();

			$session  = $this->util->getSessionVars(null);
			$products = self::getITCOMANDAVEN($session, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN);
			$CDSUPERVISOR = !empty($CDSUPERVISOR) ? $CDSUPERVISOR : $session['CDOPERADOR'];
            if (!empty($CDGRPOCORDESC)){
                $CDOCORDESC = $CDGRPOCORDESC;
                $params = array(
                    'CDFILIAL' => $session['CDFILIAL'],
                    'CDLOJA' => $session['CDLOJA'],
                    'NRORG' => $session['NRORG']
                );
                $CDGRPOCORDESC = $this->entityManager->getConnection()->fetchAssoc("SQL_GRUPO_OBS_DESC", $params);
                $CDGRPOCORDESC = !empty($CDGRPOCORDESC['CDGRPOCORDESC']) ? $CDGRPOCORDESC['CDGRPOCORDESC'] : null;
                if($CDGRPOCORDESC != null){
                    $params = array(
                        'CDGRPOCORDESC' => $CDGRPOCORDESC,
                        'CDOCORDESC' => $CDOCORDESC
                    );
                    $DSOCORRDESC = $this->entityManager->getConnection()->fetchAssoc("GET_MOT_DESC", $params);
                    if(empty($motivoDesconto)){
                        $motivoDesconto = $DSOCORRDESC["DSOCORR"] . ";";
                    } else {
                        $motivoDesconto .= "; " . $DSOCORRDESC["DSOCORR"] . ";";
                    }
                }
            }

			foreach ($products as $product) {
				$promoDiscount = 0;
				$QTPRODCOMVEN = floatval($product['QTPRODCOMVEN']);
				if($session['IDSOMADESCMANUA']=='S'){
					if(!empty($product['CDPRODPROMOCAO'])){
						$treatedProducts = array();
						$priceData = $this->precoAPI->buscaPreco($session['CDFILIAL'], $product['CDCLIENTE'], $product['CDPRODUTO'], $session['CDLOJA'], $product['CDCONSUMIDOR']);
						$productActualPrice = round($priceData['PRECO'] + $priceData['PRECOCLIE'] + $priceData['ACRE'] - $priceData['DESC'], 2);
						$promoDiscount = $this->precoAPI->promocaoAPI->calculaDesconto($session['CDFILIAL'], $product['CDPRODPROMOCAO'],$product['CDPRODUTO'], $productActualPrice, $treatedProducts);
						if($promoDiscount<0){ $promoDiscount = abs($promoDiscount); }
					}
				}
				$prodPrice = floatval($product['VRPRECCOMVEN']);
                $prodSubsidy = floatval($product['VRPRECCLCOMVEN']);
				if ($TIPODESCONTO == 'V'){
					$currentDiscount = $promoDiscount+$VRDESCONTO;
					if ($currentDiscount >= (($prodPrice + $prodSubsidy) * $QTPRODCOMVEN)){
						// verifica se o desconto esta maior que o valor dos produtos
						$currentDiscount = ($prodPrice + $prodSubsidy) * $QTPRODCOMVEN;
						if ($currentDiscount > (0.01 * $QTPRODCOMVEN)){
							$currentDiscount = $currentDiscount - (0.01 * $QTPRODCOMVEN);
						}
					}
				} else if ($TIPODESCONTO == 'P'){
					$prodPrice_temp = $prodPrice - $promoDiscount;
					$prodPrice_temp = floatval(bcmul(str_replace(',','.',strval($QTPRODCOMVEN)), str_replace(',','.',strval($prodPrice_temp)), 2));
					$currentDiscount = $this->trunc(floatval(bcdiv(str_replace(',','.',strval($VRDESCONTO)), '100', 4)) * $prodPrice_temp, 2);
					$currentDiscount = $currentDiscount + $promoDiscount;
				}
				if ($currentDiscount >= (($prodPrice+$prodSubsidy) * $QTPRODCOMVEN)){
					// verifica se o desconto esta maior que o valor dos produtos
					$currentDiscount = ($prodPrice + $prodSubsidy) * $QTPRODCOMVEN;
					if ($currentDiscount > (0.01 * $QTPRODCOMVEN)){
						$currentDiscount = $currentDiscount - (0.01 * $QTPRODCOMVEN);
					}
				}
				$paramsUpdate = $this->tableService->formatUpdateITCOMANDAVEN($product, $prodPrice, $prodSubsidy, $currentDiscount, floatval($product['VRACRCOMVEN']), $motivoDesconto, $CDGRPOCORDESC, 'S', $CDSUPERVISOR);
				$this->entityManager->getConnection()->executeQuery("UPDATE_PRICE_ON_ITCOMANDAVEN", $paramsUpdate);
				$this->util->logFOS($session['CDFILIAL'], $session['CDCAIXA'], 'CUP_DES', $session['CDOPERADOR'], $CDSUPERVISOR, 'Waiter - Cupom com desconto.', 'Desconto no cupom nº ' . $NRCOMANDA);
			}

			$connection->commit();

			$result['error'] = false;
		} catch (\Exception $e) {
			if ($connection != null){
				$connection->rollback();
			}

			$result['message'] = $e->getMessage();
		}

		return $result;
	}

	private function getITCOMANDAVEN($session, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN){
        $params = array(
            'CDFILIAL' => $session['CDFILIAL'],
            'NRORG' => $session['NRORG'],
            'NRVENDAREST' => $NRVENDAREST,
            'NRCOMANDA' => $NRCOMANDA,
            'NRPRODCOMVEN' => array_filter(array_unique($NRPRODCOMVEN))
        );
        $type = array(
            'NRPRODCOMVEN' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
        );

        return $this->entityManager->getConnection()->fetchAll("SQL_ITCOMANDAVEN_NRPRODCOMVEN", $params, $type);
    }

    private function checkClientConsumer($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR){
        if (!empty($CDCLIENTE) && empty($CDCONSUMIDOR)){
            $params = array(
                $CDFILIAL
            );
            $CLIENTE_PADRAO = $this->entityManager->getConnection()->fetchAssoc("SQL_GETCLIENTEPADRAO", $params);

            if ($CDCLIENTE !== $CLIENTE_PADRAO['CDCLIENTE']){
                throw new \Exception("Favor informar um consumidor para o cliente escolhido.");
            }
        }
    }

    public function updateCartPrices(&$products, $CDFILIAL, $CDLOJA = null, $CDCLIENTE = null, $CDCONSUMIDOR = null){

        $this->checkClientConsumer($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);

        $numeroProdutos = 0;
        foreach ($products as &$product){

            $price = $this->precoAPI->buscaPreco($CDFILIAL, $CDCLIENTE, $product['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);

            if (!empty($product['PRODUTOS']) && $product['IDIMPPRODUTO'] == '2'){
                $price['error'] = true;
            }

            if (!$price['error']){
                $product['PRITEM'] = $price['PRECO'];
                $product['PRECO'] = $price['PRECO'];
                $product['VRDESITVEND'] = $price['DESC'];
                $product['VRACRITVEND'] = $price['ACRE'];
                $product['VRPRECITEMCL'] = $price['PRECOCLIE'];
                $product['PRITOTITEM'] = floatval(bcsub(str_replace(',','.',strval($price['PRECO'] + $price['PRECOCLIE'] + $price['ACRE'])), str_replace(',','.',strval($price['DESC'])), '2'));
                if ($product['IDPESAPROD'] == 'S') $numeroProdutos++;
                else $numeroProdutos += $product['QTPRODCOMVEN'];
            }
            else {
                if (!empty($product['PRODUTOS'])){
                    if ($product['IDIMPPRODUTO'] == '2' && $product['IDTIPOCOMPPROD'] !== 'C'){
                        // Promoção Inteligente.
                        $treatedProducts = array();
                        foreach ($product['PRODUTOS'] as &$subProduct){
                            $priceData = $this->precoAPI->buscaPreco($CDFILIAL, $CDCLIENTE, $subProduct['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
                            $productActualPrice = round($priceData['PRECO'] + $priceData['PRECOCLIE'] + $priceData['ACRE'] - $priceData['DESC'], 2);
                            $promoDiscount = $this->precoAPI->promocaoAPI->calculaDesconto($CDFILIAL, $product['CDPRODUTO'], $subProduct['CDPRODUTO'], $productActualPrice, $treatedProducts);
                            $subProduct['PRECO'] = $subProduct['PRITEM'] = $priceData['PRECO'];
                            $subProduct['VRPRECITEMCL'] = $priceData['PRECOCLIE'];
                            $price = floatval(bcadd(str_replace(',','.',strval($productActualPrice)), str_replace(',','.',strval($promoDiscount)), '2'));

                            if ($promoDiscount > 0){
                                $subProduct['VRACRITVEND'] = floatval(bcadd(str_replace(',','.',strval($priceData['ACRE'])), str_replace(',','.',strval($promoDiscount)), '2'));
                            }
                            else {
                                $subProduct['VRDESITVEND'] = floatval(bcadd(str_replace(',','.',strval($priceData['DESC'])), str_replace(',','.',strval(-1 * $promoDiscount)), '2'));
                            }

                            $subProduct['PRICE'] = $subProduct['TOTPRICE'] = $price;
                            $numeroProdutos += $subProduct['QTPRODCOMVEN'];
                            array_push($treatedProducts, $subProduct['CDPRODUTO']);
                        }
                    }
                    else {
                        // Produto Combinado.
                        $maxPrice = 0;
                        foreach ($product['PRODUTOS'] as &$subProduct){
                            $priceData = $this->precoAPI->buscaPreco($CDFILIAL, $CDCLIENTE, $subProduct['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
                            $subProduct['PRICE']  = $subProduct['PRITEM'] = round($priceData['PRECO'] + $priceData['PRECOCLIE'] + $priceData['ACRE'] - $priceData['DESC'], 2);
                            if ($subProduct['PRICE'] <= 0) $subProduct['PRICE'] = 0.01;

                            if ($product['IDTIPCOBRA'] === 'C' && $subProduct['PRICE'] > $maxPrice){
                                $maxPrice = $subProduct['PRICE'];
                            }
                        }

                        foreach ($product['PRODUTOS'] as &$subProduct){
                            if ($product['IDTIPCOBRA'] === 'C'){
                                $subProduct['PRICE']  = $subProduct['PRITEM'] = $maxPrice;
                            }
                            $subProduct['PRECO']  = $subProduct['PRITEM'] = number_format($subProduct['PRICE'], 3, '.', '');
                            $subProduct['STRPRICE'] = number_format($subProduct['PRICE'], 2, ',', '.');

                            $numeroProdutos++;
                        }
                    }
                    foreach ($product['PRODUTOS'] as &$subProduct) {
                        $totalObsAcrescimo = 0;
                        foreach ($subProduct['CDOCORR'] as &$CDOCORR) {
                            $this->calculaObsAcrescimo($CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $CDOCORR, $totalObsAcrescimo, $numeroProdutos, $product['QTPRODCOMVEN']);
                        }
                        $subProduct['totalObsAcrescimo'] = $totalObsAcrescimo;
		        	}
                }
                else {
                    throw new \Exception($price['message'], 1);
                }
            }

            $totalObsAcrescimo = 0;
            if (array_key_exists('CDOCORR', $product)){
				if(is_array($product['CDOCORR'])){
		            foreach ($product['CDOCORR'] as &$CDOCORR){
		                $this->calculaObsAcrescimo($CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $CDOCORR, $totalObsAcrescimo, $numeroProdutos, $product['QTPRODCOMVEN']);
		            }
				}
			}
            $product['totalObsAcrescimo'] = $totalObsAcrescimo;
        }

        $products = $this->calculaDescontoSubgrupo($products, $CDCLIENTE, $CDCONSUMIDOR,false);

        $valorVenda = 0;
        $descontos = 0; // Controle de desconto para saber qual o valor total da venda.
        $subsidioTotal = 0;
        $subsidioReal = 0;
        foreach ($products as &$product){
            if (!empty($product['PRODUTOS']) && $product['IDIMPPRODUTO'] == '2'){
                foreach ($product['PRODUTOS'] as &$subProduct){

					$subProduct = self::calcDescontoVoucher($subProduct);

                    $valorVenda += floatval(bcmul(str_replace(',','.',strval($product['QTPRODCOMVEN'] * $subProduct['QTPRODCOMVEN'])), str_replace(',','.',strval($subProduct['PRITEM'] + $subProduct['VRPRECITEMCL'])), '2'));
					$valorVenda += $subProduct['totalObsAcrescimo'];
					//soma os descontos e somente no final subtrai do valor total da venda
					$descontos += floatval(bcmul(strval($product['QTPRODCOMVEN'] * $subProduct['QTPRODCOMVEN']), strval($subProduct['VRDESITVEND']), 2));
                    $descontos = floatval(bcadd(strval($descontos), strval($subProduct['VOUCHERDISCOUNT']), 2));

                    $subsidioTotal += floatval(bcmul(str_replace(',','.',strval($product['QTPRODCOMVEN'] * $subProduct['QTPRODCOMVEN'])), str_replace(',','.',strval($subProduct['VRPRECITEMCL'])), '2'));
                    $subProduct['REALSUBSIDY'] = self::calculateSubsidy($subProduct['PRECO'], $subProduct['VRPRECITEMCL'], $subProduct['VRDESITVEND']);
                    $subProduct['REALSUBSIDY'] = floatval(bcmul(str_replace(',','.',strval($product['QTPRODCOMVEN'] * $subProduct['QTPRODCOMVEN'])), str_replace(',','.',strval($subProduct['REALSUBSIDY'])), '2'));
                    $subsidioReal += $subProduct['REALSUBSIDY'];
                }
            }
            else if ($product['IDTIPOCOMPPROD'] === 'C'){
                foreach ($product['PRODUTOS'] as &$subProduct){
                    $total = floatval(bcmul(str_replace(',','.',strval($subProduct['PRICE'])), str_replace(',','.',strval($subProduct['QTPRODCOMVEN'])), '2'));
                    if ($total <= 0){ $total = 0.01; }
                    $valorVenda += $total + $subProduct['totalObsAcrescimo'];
                    //soma os descontos e somente no final subtrai do valor total da venda
					$descontos += floatval(bcmul(strval($subProduct['QTPRODCOMVEN']), strval($subProduct['VRDESITVEND']), 2));
                }
            } else {

				$product = self::calcDescontoVoucher($product);

                $product['QTPRODCOMVEN'] = str_replace(',','.', strval($product['QTPRODCOMVEN']));
                $product['PRECO']        = str_replace(',','.', strval($product['PRECO']));
                $product['VRPRECITEMCL'] = str_replace(',','.', strval($product['VRPRECITEMCL']));
                $product['VRACRITVEND']  = str_replace(',','.', strval($product['VRACRITVEND']));
                $product['VRDESITVEND']  = str_replace(',','.', strval($product['VRDESITVEND']));

                $valorVenda += floatval(bcmul($product['QTPRODCOMVEN'], ($product['PRECO'] + $product['VRPRECITEMCL']), 2)) + floatval(bcmul($product['QTPRODCOMVEN'], $product['VRACRITVEND'], 2));
                $valorVenda += $product['totalObsAcrescimo'];
                //soma os descontos e somente no final subtrai do valor total da venda
                $descontos += floatval(bcmul(strval($product['QTPRODCOMVEN']), strval($product['VRDESITVEND']), 2));
                $descontos = floatval(bcadd(strval($descontos), strval($product['VOUCHERDISCOUNT']), 2));

				$subsidioTotal += floatval(bcmul(str_replace(',','.',strval($product['QTPRODCOMVEN'])), str_replace(',','.',strval($product['VRPRECITEMCL'])), '2'));
                $product['REALSUBSIDY'] = self::calculateSubsidy($product['PRECO'], $product['VRPRECITEMCL'], $product['VRDESITVEND']);
                $product['REALSUBSIDY'] = floatval(bcmul(str_replace(',','.',strval($product['QTPRODCOMVEN'])), str_replace(',','.',strval($product['REALSUBSIDY'])), '2'));
                $subsidioReal += $product['REALSUBSIDY'];
            }
        }
		return array(
            'valorVenda' => floatval(bcsub(strval($valorVenda), strval($descontos), 2)),
            'totalVenda' => floatval(bcmul(str_replace(',','.',strval($valorVenda)), '1', '2')),
            'subsidioTotal' => floatval(bcmul(str_replace(',','.',strval($subsidioTotal)), '1', '2')),
            'subsidioReal' => floatval(bcmul(str_replace(',','.',strval($subsidioReal)), '1', '2')),
            'numeroProdutos' => $numeroProdutos
        );
    }

	public function calcDescontoVoucher($product){
		if (!empty($product['VOUCHER'])){
			$params = array(
				'CDCUPOMDESCFOS' => $product['VOUCHER']['CDCUPOMDESCFOS']
			);
			$voucherData = $this->entityManager->getConnection()->fetchAssoc("SQL_VOUCHER_DATA", $params);

			$total = floatval(bcmul(strval($product['PRECO'] + $product['VRPRECITEMCL'] + $product['VRACRITVEND'] - $product['VRDESITVEND']), strval($product['QTPRODCOMVEN']), 2));
			if ($voucherData['IDTIPODESC'] === "P"){
				$voucherDiscount = bcmul(strval($total), strval($voucherData['VRDESCCUPOM']/100), 2);
			} else {
				$voucherDiscount = $voucherData['VRDESCCUPOM'];
			}

			$product['VOUCHERDISCOUNT'] = floatval($voucherDiscount);
			if ($product['VOUCHERDISCOUNT'] >= $total){
				$product['VOUCHERDISCOUNT'] = floatval(bcsub(strval($product['QTPRODCOMVEN'] * ($product['PRECO'] + $product['VRPRECITEMCL'] + $product['VRACRITVEND'])), strval($product['QTPRODCOMVEN'] * 0.01), 2));
			}
		} else {
			$product['VOUCHERDISCOUNT'] = 0;
		}
		return $product;
	}

    private function calculaObsAcrescimo($CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $CDOCORR, &$totalObsAcrescimo, &$numeroProdutos, $QTPRODCOMVEN){
        $params = array(
            $CDFILIAL,
            $CDLOJA,
            $CDOCORR
        );
        $observation = $this->entityManager->getConnection()->fetchAssoc("SQL_GET_OBS_TYPE", $params);

        // Verifica se a observação é um acrescimo.
        if ($observation['IDCONTROLAOBS'] === 'A' && !empty($observation['CDPRODUTO'])) {
            // Busca o preço do acréscimo.
            $price = $this->precoAPI->buscaPreco($CDFILIAL, $CDCLIENTE, $observation['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
            if (!$price['error']){
                $totalObsAcrescimo += floatval(bcmul(str_replace(',','.',strval($QTPRODCOMVEN)), str_replace(',','.',strval(number_format(($price['PRECO'] + $price['PRECOCLIE']) - $price['DESC'] + $price['ACRE'], 2, '.', ''))), '2'));
                $numeroProdutos++;
            } else {
                throw new \Exception($price['message'], 1);
            }
        }
    }

    private function calculateSubsidy($VRPRECITEM, $VRPRECITEMCL, $VRDESITVEND){
        if ($VRPRECITEMCL > 0 && $VRDESITVEND > 0){

            $finalPrice = floatval(bcsub(str_replace(',','.',strval($VRPRECITEM + $VRPRECITEMCL)), str_replace(',','.',strval($VRDESITVEND)), '2'));
            if ($finalPrice <= 0.01) return 0;

            $totalPrice = $VRPRECITEM + $VRPRECITEMCL;
            $discount = ($VRPRECITEMCL / $totalPrice) * $VRDESITVEND;

            return floatval(bcsub(str_replace(',','.',strval($VRPRECITEMCL)), str_replace(',','.',strval($discount)), '2'));
        }
        else return $VRPRECITEMCL;
    }

    public function consumerSearch($CDFILIAL, $CDCLIENTE, $code){
    	$CDCLIENTE = !empty($CDCLIENTE) ? $CDCLIENTE : 'T';

        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'code' => strtoupper(strval($code))
        );
        if ($this->util->databaseIsOracle()){
        	$params['code'] = strval($code);
        }

        $consumer = $this->entityManager->getConnection()->fetchAll("SQL_CONSUMER_ALL_FIELDS", $params);
        if (empty($consumer)){
	        $consumer = $this->entityManager->getConnection()->fetchAll("SQL_CONSUMER_CDEXCONSUMID", $params);
	        if (empty($consumer)){
	            $params['code'] = str_pad($code, 10, '0', STR_PAD_LEFT);
	            $consumer = $this->entityManager->getConnection()->fetchAll("SQL_CONSUMER_CDIDCONSUMID", $params);
	            if (empty($consumer)){
	                $params['code'] = $code;
	                $consumer = $this->entityManager->getConnection()->fetchAll("SQL_GET_CONSUMER_DETAILS", $params);
	                if (empty($consumer)){
	                    $consumer = $this->entityManager->getConnection()->fetchAll("SQL_CONSUMER_CDIDCONSUMID", $params);
	                    if (empty($consumer)){
	                        $consumer = $this->entityManager->getConnection()->fetchAll("SQL_CONSUMER_NRCPFRESPCON", $params);
	                        if (empty($consumer)){
		                        $consumer = $this->entityManager->getConnection()->fetchAll("SQL_CONSUMER_NRCPFRESPCON", $params);
		                    }
	                    }
	                }
	            }
	        }
	    }

        return $consumer;
    }

    public function getIDTPVENDACONS($CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'code' => $CDCONSUMIDOR
        );
        $consumer = $this->entityManager->getConnection()->fetchAssoc("SQL_GET_CONSUMER_DETAILS", $params);
        $result = !empty($consumer) ? $consumer['IDTPVENDACONS'] : null;
        return  $result;
    }

    public function verificaProdutosBloqueados($session, $produtos){
		$prodpai = array_column($produtos, 'PRODUTOS');
		$prodfilho = array();
		for ($i = 0; $i < Count($prodpai); $i++) {
			if (!empty($prodpai[$i])){
				$prodfilho = array_merge($prodfilho, $prodpai[$i]);
			}
		}
		$params = array(
            'CDFILIAL' => $session['CDFILIAL'],
			'CDLOJA' => $session['CDLOJA'],
			'NRORG' => $session['NRORG'],
            'FILIALVIGENCIA' => $session['FILIALVIGENCIA'],
			'NRCONFTELA' => $session['NRCONFTELA'],
            'DTINIVIGENCIA' => new \DateTime($session['DTINIVIGENCIA']),
			'CDPRODUTO' => array_unique(array_merge(array_column($prodfilho, 'CDPRODUTO'), array_column($produtos, 'CDPRODUTO')))
		);
		$type = array(
			'CDPRODUTO' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY,
            'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
		);

        return $this->entityManager->getConnection()->fetchAll("BUSCA_NOMEPRODBLOQ", $params, $type);
    }

    public function searchCard($searchValue){
        $session = $this->util->getSessionVars(null);

        $params = array(
            'SEARCH_VALUE' => $searchValue
        );

        if ($session['IDEXTCONSONLINE'] !== 'S'){
            $result = $this->entityManager->getConnection()->fetchAll("SQL_BUSCA_CARTOES", $params);
        }
        else {
            $result = $this->extratocons->consultaCartao($params['SEARCH_VALUE']);
            if (!empty($result)){
                $result = $result[0];
                if (!array_key_exists(0, $result)){
                    $result = array(0 => $result);
                }
            }

            foreach ($result as &$consumer){
                if (is_array($consumer['CDFAMILISALD'])) $consumer['CDFAMILISALD'] = null;
                if (is_array($consumer['NMFAMILISALD'])) $consumer['NMFAMILISALD'] = null;
                $consumer['ID'] = $consumer['CDCLIENTE']."-".$consumer['CDCONSUMIDOR']."-".$consumer['CDFAMILISALD'];
                $consumer['IDSITCONSUMI'] = '1';
            }
        }

        return $result;
    }

	public function filterProducts($session, $filter, $FIRST, $LAST, $campanha){
		$CDFILIAL = $session['CDFILIAL'];
		$CDLOJA = $session['CDLOJA'];
        $FILIALVIGENCIA = $session['FILIALVIGENCIA'];
		$NRCONFTELA = $session['NRCONFTELA'];
        $DTINIVIGENCIA = new \DateTime($session['DTINIVIGENCIA']);
		$now = $this->date->getDataAtual("d/m/Y");

		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDLOJA' => $CDLOJA,
            'FILIALVIGENCIA' => $FILIALVIGENCIA,
			'NRCONFTELA' => $NRCONFTELA,
            'DTINIVIGENCIA' => $DTINIVIGENCIA,
			'DATAATUAL' => $now,
			'FIRST' => $FIRST,
			'LAST' => $LAST,
			'SEARCH' => $filter[0] == '%' ? 0 : 1
		);

		$types = array(
            'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME,
			'DATAATUAL' => \Doctrine\DBAL\TypeS\Type::DATE
		);

        if ($campanha){
            $produtosCampanha = $this->parametrosAPI->buscaCampanhaCompreGanhe($CDFILIAL);
            $produtos = array();
            if (!empty($produtosCampanha)){
            	foreach ($produtosCampanha as $produtosCampanhaAtual){
                	$params['CDCAMPCOMPGANHE'] = $produtosCampanhaAtual['CDCAMPCOMPGANHE'];
                	$params['FILTER'] = strtoupper($filter);
                	$params['DTINIVGCAMPCG'] = new \DateTime($produtosCampanhaAtual['DTINIVGCAMPCG']);
                	$types['DTINIVGCAMPCG'] =  \Doctrine\DBAL\Types\Type::DATETIME;
                	foreach ($produtosCampanhaAtual['products'] as $prod){
                		$params['CDPRODUTO'] = $prod['CDPRODUTO'];

                		$produtos = $this->entityManager->getConnection()->fetchAll('FILTRAR_PRODUTOS_CAMPANHA', $params, $types);
                	}
            	}
            }
        } else if(is_array($filter[0])){
        	foreach ($filter[0] as $filtro){
        		$params['FILTER'] = strtoupper($filtro);
            	$produtos = $this->entityManager->getConnection()->fetchAll('FILTRAR_PRODUTOS', $params, $types);
            	if($produtos){
            		break;
            	}
        	}
        } else {
        	$params['FILTER'] = strtoupper($filter);
            $produtos = $this->entityManager->getConnection()->fetchAll('FILTRAR_PRODUTOS', $params, $types);
        }

		if ($produtos){
			$precos = $this->parametrosAPI->buscaTabelaPadrao($CDFILIAL, $CDLOJA, $CDLOJA, $produtos);
			if (!$precos['error']) {
				$obsIndexadasPorProduto = $this->parametrosAPI->montaObservacoes($CDFILIAL, $CDLOJA);
				$produtos = array_values($precos['precosIndexadosPorProduto']);
				foreach ($produtos as &$produto) {
					$produto['CDARVPROD'] = $this->addMaskCDARVPROD($produto['CDARVPROD']);
					$produto['GRUPOS'] = array();
					$produto['PRODUTOS'] = array();
					$produto['PRITEM'] = floatval($produto['PRITEM']);
					$produto['VRPRECITEMCL'] = floatval($produto['VRPRECITEMCL']);
					$produto['HRFIMVENPROD'] = floatval($produto['HRFIMVENPROD']);
					$produto['HRINIVENPROD'] = floatval($produto['HRINIVENPROD']);
					$produto['PRECO'] = $this->util->formataPreco($produto['PRECO']);
					$produto['OBSERVATIONS'] = array_key_exists($produto['CDPRODUTO'], $obsIndexadasPorProduto)
						? $obsIndexadasPorProduto[$produto['CDPRODUTO']] : array();
				}
			} else {
				throw new \Exception($precos['message']);
			}
		}
		return array_values($produtos);
	}

	private function addMaskCDARVPROD($CDARVPROD) {
		$CDARVPROD = substr($CDARVPROD, 0, 1) . '.' . substr($CDARVPROD, 1);
		$CDARVPROD = substr($CDARVPROD, 0, 4) . '.' . substr($CDARVPROD, 4);
		$CDARVPROD = substr($CDARVPROD, 0, 7) . '.' . substr($CDARVPROD, 7);
		$CDARVPROD = substr($CDARVPROD, 0, 11) . '.' . substr($CDARVPROD, 11);
		return $CDARVPROD;
	}

    public function transferPosition($NRVENDAREST, $NRCOMANDA, $products, $position, $CDCLIENTE, $CDCONSUMIDOR){
        try {
            $session = $this->util->getSessionVars(null);

            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            // Valida se a posição está sendo recebida.
            $positionControl = $this->tableService->getPositionControlDetails($session['CDFILIAL'], $NRVENDAREST, $session['CDOPERADOR']);
            foreach ($positionControl as $lockedPosition){
                if (intval($lockedPosition) == $position){
                    throw new \Exception("Não foi possível realizar a transferência. Esta posição está sendo recebida.");
                }
            }

            foreach ($products as $NRPRODCOMVEN){

                $params = array(
                    'NRVENDAREST' => $NRVENDAREST,
                    'NRCOMANDA' => $NRCOMANDA,
                    'NRPRODCOMVEN' => $NRPRODCOMVEN
                );
                $positionDetails = $this->entityManager->getConnection()->fetchAssoc("GET_POSITION_CLIENT", $params);

                // Valida se o cliente/consumidor são diferentes.
                if ($positionDetails['CDCLIENTE'] != $CDCLIENTE || $positionDetails['CDCONSUMIDOR'] != $CDCONSUMIDOR){
                    throw new \Exception("Não é possível transferir produtos para posições que possuam cliente/consumidor diferente da posição de origem.");
                }

                $params = array(
                    'POSITION' => str_pad($position, 2, '0', STR_PAD_LEFT),
                    'NRVENDAREST' => $NRVENDAREST,
                    'NRCOMANDA' => $NRCOMANDA,
                    'NRPRODCOMVEN' => $NRPRODCOMVEN
                );
                $this->entityManager->getConnection()->fetchAssoc("TRANSFER_POSITION", $params);

            }

            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            throw new \Exception($e->getMessage());
        }
    }

	public function selectVendedores($session, $filter, $FIRST, $LAST){
		$params = array(
			'CDVENDEDOR' => 'T',
			'FILTER' => strtoupper($filter),
			'FIRST' => $FIRST,
			'LAST' => $LAST,
            'CDFILIAL' => $session['CDFILIAL']
		);

		$result = $this->entityManager->getConnection()->fetchAll('VENDEDORES_OPERADORES_ATIVOS', $params);

		return array_values($result);
	}

	public function validatePassword($session, $password){
		$params = array(
            'CDOPERADOR' => $session['CDOPERADOR']
        );
        $operador = $this->entityManager->getConnection()->fetchAssoc("VALIDA_OPERADOR", $params);

		$passwordResult = $this->utilAPI->validaSenha($password, $operador['CDSENHAOPERWEB']);
        if (!$passwordResult) throw new \Exception('Senha inválida.');

		return array('error' => false, 'message' => 'OK');
	}

	public function selectComandaProducts($CDFILIAL, $NRCOMANDA){
		$params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRCOMANDA' => $NRCOMANDA
        );
        $result = $this->entityManager->getConnection()->fetchAll('SQL_COMANDA_ITENS', $params);

        if (!$result) throw new \Exception('Comanda sem pedidos lançados.');

		return array_values($result);
	}

	public function updateComandaProducts($session, $comandaAtual, $vendaRestComandaAtual, $comandaDestino, $vendaRestComandaDestino, $CDPRODUTO, $NRPRODCOMVEN, $CDSUPERVISOR){
		try {
        	$this->entityManager->getConnection()->beginTransaction();
	        $produtosLog = '';

			$params = array(
	            'CDFILIAL' => $session['CDFILIAL'],
	            'NRCOMANDA' => $comandaDestino
	        );

	        for ($i = 0; $i < count($CDPRODUTO); $i++) {
	        	// Geração de um novo código de controle.
	        	$this->util->newCode('ITCOMANDAVEN' . $session['CDFILIAL'] . $comandaDestino);
				$maxNRPRODCOMVEN = $this->util->getNewCode('ITCOMANDAVEN' . $session['CDFILIAL'] . $comandaDestino, 6);

		        // Manipulação das tabelas itcomandaest e obsitcomandaest na transferência.
				$antigo = array(
					'NRVENDAREST' => $vendaRestComandaAtual,
					'NRCOMANDA' => $comandaAtual
				);
				$novo = array(
					'NRVENDAREST' => $vendaRestComandaDestino,
					'NRCOMANDA' => $comandaDestino,
					'NRPRODCOMVEN' => str_pad($maxNRPRODCOMVEN, 6, "0", STR_PAD_LEFT)
				);
				$this->tableService->alteraComandaEst($antigo, $novo, $NRPRODCOMVEN[$i]);

				// Preparação dos parâmetros para transferência nas tabelas itcomandaven e obsitcomandaven.
		        $params = array(
		            'CDFILIAL' => $session['CDFILIAL'],
		            'NRCOMANDA' => $comandaAtual,
		            'NRVENDAREST' => $vendaRestComandaAtual,
		            'CDPRODUTO' => $CDPRODUTO[$i],
		            'NRPRODCOMVEN' => $NRPRODCOMVEN[$i],
		            'CDSUPERVISOR' => $CDSUPERVISOR,
		            'COMANDADEST' => $comandaDestino,
		            'VRCOMANDADEST' => $vendaRestComandaDestino,
		            'MAXNRPRODCOMVEN' => str_pad($maxNRPRODCOMVEN, 6, "0", STR_PAD_LEFT)
		        );

		        $produtosLog .= $produtosLog == '' ? $CDPRODUTO[$i] : ', ' . $CDPRODUTO[$i];

		        // Captura dos dados de transferência.
		        $dadosProdutos = $this->entityManager->getConnection()->fetchAll('SQL_ITCOMANDAVEN_NRPRODCOMVEN', $params);
		        $dadosObsitcomandaven = $this->entityManager->getConnection()->fetchAll('SQL_GET_OBSERVATIONS_VEN', $params);


				// Inserção dos dados nas tabelas.
		        foreach ($dadosProdutos as $produto) {
		        	$produto['NRCOMANDA'] = $comandaDestino;
		        	$produto['NRVENDAREST'] = $vendaRestComandaDestino;
		        	$produto['NRPRODCOMVEN'] = str_pad($maxNRPRODCOMVEN, 6, "0", STR_PAD_LEFT);

		        	if ($this->utilAPI->databaseIsOracle()){
	                    $produto['DTHRINCOMVEN'] = \DateTime::createFromFormat('Y-m-d H:i:s', $produto['DTHRINCOMVEN']);
	                }
	                else {
	                    $produto['DTHRINCOMVEN'] = \DateTime::createFromFormat('Y-m-d H:i:s.u', $produto['DTHRINCOMVEN']);
	                }
                    $params = array(
                        'CDFILIAL' => $produto['CDFILIAL'],
                        'NRVENDAREST' => $produto['NRVENDAREST'],
                        'NRCOMANDA' => $produto['NRCOMANDA'],
                        'NRPRODCOMVEN' => $produto['NRPRODCOMVEN'],
                        'CDPRODUTO' => $produto['CDPRODUTO'],
                        'QTPRODCOMVEN' => $produto['QTPRODCOMVEN'],
                        'VRPRECCOMVEN' => $produto['VRPRECCOMVEN'],
                        'TXPRODCOMVEN' => $produto['TXPRODCOMVEN'],
                        'IDSTPRCOMVEN' => $produto['IDSTPRCOMVEN'],
                        'VRDESCCOMVEN' => $produto['VRDESCCOMVEN'],
                        'NRLUGARMESA' => $produto['NRLUGARMESA'],
                        'NRMESAORIG' => $produto['NRMESAORIG'],
                        'CDLOJAORIG' => $produto['CDLOJAORIG'],
                        'DTHRINCOMVEN' => $produto['DTHRINCOMVEN'],
                        'IDPRODIMPFIS' => $produto['IDPRODIMPFIS'],
                        'CDLOJA' => $produto['CDLOJA'],
                        'NRSEQPRODCOM' => $produto['NRSEQPRODCOM'],
                        'NRSEQPRODCUP' => $produto['NRSEQPRODCUP'],
                        'VRACRCOMVEN' => $produto['VRACRCOMVEN'],
                        'DSCOMANDAORI' => $produto['DSCOMANDAORI'],
                        'NRCOMANDAORI' => $produto['NRCOMANDAORI'],
                        'NRPRODCOMORI' => $produto['NRPRODCOMORI'],
                        'CDCAIXACOLETOR' => $produto['CDCAIXACOLETOR'],
                        'VRPRECCLCOMVEN' => $produto['VRPRECCLCOMVEN'],
                        'CDPRODPROMOCAO' => $produto['CDPRODPROMOCAO'],
                        'CDVENDEDOR' => $produto['CDVENDEDOR'],
                        'NRPEDIDOFOS' => $produto['NRPEDIDOFOS'],
                        'CDFILIALPED' => $produto['CDFILIALPED'],
                        'CDSENHAPED' => $produto['CDSENHAPED'],
                        'NRATRAPRODCOVE' => $produto['NRATRAPRODCOVE'],
                        'IDORIGPEDCMD' => $produto['IDORIGPEDCMD'],
                        'DSOBSPEDDIGCMD' => $produto['DSOBSPEDDIGCMD'],
                        'IDPRODREFIL' => $produto['IDPRODREFIL'],
                        'QTITEMREFIL' => $produto['QTITEMREFIL'],
                        'IDDIVIDECONTA' => $produto['IDDIVIDECONTA'],
                        'CDSUPERVISOR' => $produto['CDSUPERVISOR'],
                        'CDCUPOMDESCFOS' => $produto['CDCUPOMDESCFOS']
                    );
                    $types = array(
                        'DTHRINCOMVEN' => \Doctrine\DBAL\Types\Type::DATETIME
                    );
                    $this->entityManager->getConnection()->executeQuery("SQL_INSERE_ITEM_COMANDA_VEN", $params, $types);
                }

		        foreach ($dadosObsitcomandaven as $obs) {
		        	$obs['NRCOMANDA'] = $comandaDestino;
		        	$obs['NRVENDAREST'] = $vendaRestComandaDestino;
		        	$obs['NRPRODCOMVEN'] = str_pad($maxNRPRODCOMVEN, 6, "0", STR_PAD_LEFT);

		        	$this->entityManager->getConnection()->executeQuery("SQL_INS_OBSITCOMANDAVEN", $obs);
		        }

		        // Deleção dos dados das tabelas.
		        $paramsDelItcomandaven = array($session['CDFILIAL'], $vendaRestComandaAtual, $comandaAtual, $NRPRODCOMVEN[$i]);
				$this->entityManager->getConnection()->executeQuery("SQL_DEL_PROD", $paramsDelItcomandaven);
				$this->entityManager->getConnection()->executeQuery("SQL_DELETE_OBSITCOMANDAVEN", $params);

			}

			$this->util->logFOS($session['CDFILIAL'], $session['CDCAIXA'], 'TRA_PRO', $session['CDOPERADOR'], $CDSUPERVISOR,
				"Waiter - Transferência de Produto", "Transferência de produtos da comanda " . $comandaAtual . " para comanda " .
				$comandaDestino . ". Produtos: " . $produtosLog . ".");

			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollBack();
			throw $e;
		}
	}

    public function setDiscountFidelity($NRVENDAREST, $NRCOMANDA, $CPF4ALL, $CODPROMO, $DESCONTOFIDEL, $cashback, $TELCRMBONUS, $VRDESCCRMBONUS){
        $session = $this->util->getSessionVars(null);
		$connection = null;
		
		$CDFILIAL = $session['CDFILIAL'];

        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $table = array(
                'NRVENDAREST' => $NRVENDAREST,
                'NRCOMANDA' => $NRCOMANDA
            );
            $paramsUpdate = $this->formatUpdateTableData($CDFILIAL, $table, null, null, $CPF4ALL, $CODPROMO, $DESCONTOFIDEL, $cashback, $TELCRMBONUS, $VRDESCCRMBONUS);
			$this->entityManager->getConnection()->executeQuery("UPDATE_COMANDAVEN_DESCFID", $paramsUpdate);
			$connection->commit();
        } catch(\Exception $e){
            if ($connection != null) {
                $connection->rollback();
            }
            throw $e;
        }
    }

	public function adjustDiscountFidelity($cdfilial, $nrvendarest, $produtosParcial, $cdprodcouver, $cdprodconsum, $totalSemDesconto, $totalDesconto, $numeroProdutos, $arrayPosicoes, $VRDESCPROMOFIDEL, $VRDESCCASHFIDEL, &$fidelityValue, &$cashbackUsed4ALL, &$fidelityDiscount, &$totalProdutosTaxa, &$totalSubsidy, $updateDiscount){
    	if (($VRDESCPROMOFIDEL > 0 || $VRDESCCASHFIDEL > 0) && $numeroProdutos > 0){

            $total = round($totalSemDesconto - $totalDesconto, 2);
            $minCost = floatval(0.01 * $numeroProdutos);

            // Calcula o desconto do código promocional.
            if ($updateDiscount){
    			$maxDiscount = round($total - $minCost, 2);

    			if ($VRDESCPROMOFIDEL >= $maxDiscount){
    				$fidelityDiscount = ($VRDESCPROMOFIDEL > $total) ? $total : $VRDESCPROMOFIDEL;
    				$fidelityValue = $maxDiscount;
    			} else {
    				$fidelityDiscount = $VRDESCPROMOFIDEL;
    				$fidelityValue = $VRDESCPROMOFIDEL;
    			}

                $NEWTOTAL = floatval(bcsub(strval($total), strval($fidelityValue), 2));
                if ($VRDESCCASHFIDEL > 0 && $NEWTOTAL > $minCost){
                    $maxDiscount = round($NEWTOTAL - $minCost, 2);

                    if ($VRDESCCASHFIDEL >= $NEWTOTAL){
                        $cashbackUsed4ALL = $NEWTOTAL;
                        $VRDESCCASHFIDEL = $maxDiscount;
                    }
                    else {
                        $cashbackUsed4ALL = $VRDESCCASHFIDEL;
                    }
                    $fidelityValue += $VRDESCCASHFIDEL;
                }

                // rateia desconto nos produtos
                $vrTotDescCalc = 0;
                $arrProdDesc = array();
                foreach ($produtosParcial as $key => $item){
                    if ($item['CDPRODUTO'] != $cdprodcouver && $item['CDPRODUTO'] != $cdprodconsum){
                        $vrProd = $this->util->calculaTotalItem($item);
                        $valorRateio = bcdiv(strval($vrProd), strval($total), 10);
                        $desc = floatval(bcmul($valorRateio, strval($fidelityValue), 2));
                        $item['VRDESCCOMVEN'] += $desc;
                        $arrProdDesc[$key] = $item;
                        $vrTotDescCalc += $desc;
                    }
                }
                if ($vrTotDescCalc < $fidelityValue){
                    $this->util->adjustDifference($arrProdDesc, round($fidelityValue - $vrTotDescCalc, 2), 'prodObject', 'VRDESCCOMVEN');
                }

                // inclui ao array principal de produtos o rateio do desconto
                foreach ($arrProdDesc as $key => $item) {
                    $produtosParcial[$key] = $item;
                    // Atualiza ITCOMANDAVEN com os descontos.
                    $paramsUpdate = $this->tableService->formatUpdateITCOMANDAVEN($item, $item['VRPRECCOMVEN'], $item['VRPRECCLCOMVEN'], $item['VRDESCCOMVEN'], $item['VRACRCOMVEN'], $item['DSOBSDESCIT'], $item['CDGRPOCORDESCIT'], 'N');
                    $this->entityManager->getConnection()->executeQuery("UPDATE_PRICE_ON_ITCOMANDAVEN", $paramsUpdate);
                }

    			// recalcula preço dos produtos para a taxa de serviço
    			$totalProdutosTaxa = 0;
    			$totalProdutosTaxaDesc = 0;
    			$totalSemDesconto = 0;
    			$totalDesconto = 0;
                $totalSubsidy = 0;
    			self::calculaValoresParcial($cdfilial, $produtosParcial, $cdprodcouver, $cdprodconsum, $totalProdutosTaxa, $totalProdutosTaxaDesc, $totalSemDesconto, $totalDesconto, $totalSubsidy);
    			$totalProdutosTaxa = round(round($totalProdutosTaxa, 2) - $totalProdutosTaxaDesc, 2);
            }
            else {
                // Temos que levar em consideração que se o desconto fidelidade já tiver sido aplicado anteriormente, o desconto total já inclui descontos do código promocional e/ou do cashback
                if (!$updateDiscount){
                    $fidelityValue = floatval(bcadd(strval($VRDESCPROMOFIDEL), strval($VRDESCCASHFIDEL), 2));
                    if ($fidelityValue >= $totalSemDesconto){
                        $fidelityValue = $totalDesconto;
                    }
                    $fidelityDiscount = $VRDESCPROMOFIDEL;
                    $cashbackUsed4ALL = $VRDESCCASHFIDEL;
                    // Ajuste para garantir que o valor do fidelidade leve em consideração o preço mínimo, caso existam outros descontos.
                    $otherDiscounts = floatval(bcsub(strval($totalSemDesconto), strval($fidelityValue), 2));
                    $fullDiscounts = floatval(bcadd(strval($otherDiscounts), strval($fidelityValue), 2));
                    if ($fullDiscounts == $totalSemDesconto){
                        $fidelityValue = floatval(bcsub(strval($fidelityValue), strval($minCost), 2));
                    }
                }
            }
    	}
	}

	public function fidelitySearch($session, $CDCLIENTE, $CDCONSUMIDOR){
		$result = array();

		try {
		    $saldo = $this->consumidorAPI->buscaCreditoFidelidade($session['CDFILIAL'], $CDCLIENTE, $CDCONSUMIDOR, $session['NRORG']);
	        $dadosTaxa = $this->caixaAPI->buscaDadosTaxa($session['CDFILIAL'], $session['CDLOJA']);

	        $result = array(
	            'VRSALDCONEXT' => $saldo['VRSALDCONEXT'],
	            'IDPERALTDESCFID' => $saldo['IDPERALTDESCFID'],
	            'IDCOMISVENDA' => $dadosTaxa['IDCOMISVENDA'],
	            'VRCOMISVENDA' => floatval($dadosTaxa['VRCOMISVENDA'])
	        );
		} catch (\Exception $e) {
			// erro ainda não tratado
			Exception::logException($e);
		}

		return $result;
	}

    public function calculaDescontoSubgrupo($produtos, $CDCLIENTE = null, $CDCONSUMIDOR = null, $tratarcampanha = true){
        $productsParam = array();
        foreach($produtos as $key => &$produto){
            if (!empty($produto['PRODUTOS']) && $produto['IDIMPPRODUTO'] == '2' && $produto['IDTIPOCOMPPROD'] !== 'C'){
                foreach($produto['PRODUTOS'] as $subKey => &$subProduto){
                    $index = $key.':'.$subKey;
                    $productsParam[$index] = array(
                        'INDEX'      => $index,
                        'CDPRODUTO'  => $subProduto['CDPRODUTO'],
                        'QUANTIDADE' => floatval(bcmul(str_replace(',','.',strval($produto['QTPRODCOMVEN'])), str_replace(',','.',strval($subProduto['QTPRODCOMVEN'])), '3')),
                        'VALOR'      => $subProduto['PRITEM'] + $subProduto['VRPRECITEMCL'] + $subProduto['VRACRITVEND'],
                        'DESCONTO'   => floatval($subProduto['VRDESITVEND'])
                    );
                    $productsParam[$index]['TOTAL'] = floatval(bcsub(str_replace(',','.',strval($productsParam[$index]['VALOR'])), str_replace(',','.',strval($productsParam[$index]['DESCONTO'])), '2'));
                    if ($productsParam[$index]['TOTAL'] <= 0.01){
                        $productsParam[$index]['DESCONTO'] = $productsParam[$index]['VALOR'] - 0.01;
                        $productsParam[$index]['TOTAL'] = 0.01;
                    }
                }
            }
            else {
                $productsParam[$key] = array(
                    'INDEX'      => $key,
                    'CDPRODUTO'  => $produto['CDPRODUTO'],
                    'QUANTIDADE' => $produto['QTPRODCOMVEN'],
                    'VALOR'      => $produto['PRITEM'] + $produto['VRPRECITEMCL'] + $produto['VRACRITVEND'],
                    'DESCONTO'   => floatval($produto['VRDESITVEND'])
                );
                $productsParam[$key]['TOTAL'] = floatval(bcsub(str_replace(',','.',strval($productsParam[$key]['VALOR'])), str_replace(',','.',strval($productsParam[$key]['DESCONTO'])), '2'));
                if ($productsParam[$key]['TOTAL'] <= 0.01){
                    $productsParam[$key]['DESCONTO'] = $productsParam[$key]['VALOR'] - 0.01;
                    $productsParam[$key]['TOTAL'] = 0.01;
                }
                $productsParam[$key]['ADICIONAIS'] = floatval(bcsub(str_replace(',','.',strval($produto['PRITOTITEM'])), str_replace(',','.',strval($productsParam[$key]['TOTAL'])), '2'));
            }
        }

        $descontosSubgrupo = $this->precoAPI->applySubgroupDiscount($productsParam);

        foreach ($descontosSubgrupo as $key => &$produto){
            if (strrpos($key, ":")){
                $s = explode(":", $key);
                $produtos[$s[0]]['PRODUTOS'][$s[1]]['VRDESITVEND'] = floatval(bcmul(str_replace(',','.',strval($produto['DESCONTO'])), '1', '2'));
                $produtos[$s[0]]['PRODUTOS'][$s[1]]['PRICE'] = $produto['TOTAL'];
                $produtos[$s[0]]['PRODUTOS'][$s[1]]['TOTPRICE'] = $produto['TOTAL'];
                $produtos[$s[0]]['PRODUTOS'][$s[1]]['REALPRICE'] = $produto['TOTAL'];
            }
            else {
                $produtos[$produto['INDEX']]['VRDESITVEND'] = floatval(bcmul(str_replace(',','.',strval($produto['DESCONTO'])), '1', '2'));
                $produtos[$produto['INDEX']]['PRITOTITEM'] = $produto['TOTAL'] + $produto['ADICIONAIS'];
            }
        }

        // Ajusta preço de promoções.
        foreach ($produtos as &$produto){
            if (!empty($produto['PRODUTOS']) && $produto['IDIMPPRODUTO'] == '2' && $produto['IDTIPOCOMPPROD'] !== 'C'){
                $totalPrice = 0;
                $VRDESITVEND = 0; // Reflexão dos descontos dos produtos para o produto pai.
                foreach($produto['PRODUTOS'] as &$subProduto){
                    $subProductPrice = floatval(bcmul(str_replace(',','.',strval($subProduto['QTPRODCOMVEN'])), str_replace(',','.',strval($subProduto['PRICE'])), '2'));
                    $subProductDiscount = floatval(bcmul(strval($subProduto['QTPRODCOMVEN']), strval($subProduto['VRDESITVEND']), 2));
                    // Aplica o desconto do voucher.
                    if (!empty($subProduto['VOUCHER'])){
                        if ($subProduto['VOUCHER']['IDTIPODESC'] === "P"){
                            $voucherDiscount = $subProductPrice * $subProduto['VOUCHER']['VRDESCCUPOM'] / 100;
                        }
                        else {
                            $voucherDiscount = $subProduto['VOUCHER']['VRDESCCUPOM'];
                        }
                        $subProduto['VOUCHERDISCOUNT'] = floatval(bcmul(strval($voucherDiscount), "1", 2));
                        if ($subProduto['VOUCHERDISCOUNT'] >= $subProductPrice) $subProduto['VOUCHERDISCOUNT'] = $subProductPrice - 0.01 * $subProduto['QTPRODCOMVEN'];
                        $subProductPrice = floatval(bcsub(strval($subProductPrice), strval($subProduto['VOUCHERDISCOUNT']), 2));
                    }
                    $VRDESITVEND = floatval(bcadd(strval($VRDESITVEND), strval($subProductDiscount + $subProduto['VOUCHERDISCOUNT']), 2));
                    $totalPrice += $subProductPrice;
                }
                $produto['PRITOTITEM'] = $totalPrice + $produto['EXTRAS'];
                $produto['VRDESITVEND'] = $VRDESITVEND;
            }
        }

		if($tratarcampanha){ // como o desconto já esta sendo enviado pelo front, deve ignorar para que não duplique o desconto
        	// Aplica desconto da campanha promocional.
			$produtos = self::recalculoCampanha($CDCLIENTE, $CDCONSUMIDOR, $produtos);
		}
        return $produtos;
    }

	private function recalculoCampanha($CDCLIENTE, $CDCONSUMIDOR, $promocoes){
		foreach ($promocoes as $keyCombo => $combo){
			$devolveItens = false;
			if(!empty($combo['PRODUTOS']) || $combo['IDTIPOCOMPPROD'] == '3' ){
				// procura os itens do combo que estão fora do combo e coloca dentro do combo para fazer a busca da campanha
				// é nescessário fazer isso pois a função de busca da campanha precisa de todos os itens do combo incluindo aqueles que estão fora (impriem sim)
				foreach ($promocoes as $keyOriginal => $procuraItensCombo){
					if(isset($procuraItensCombo['CDPRODPROMOCAO'])){
						if($procuraItensCombo['CDPRODPROMOCAO'] == $combo['CDPRODUTO'] && $procuraItensCombo['idComboProduto'] == $combo['idComboProduto']){
							$procuraItensCombo['keyOriginal'] = $keyOriginal;
							array_push($combo['PRODUTOS'],$procuraItensCombo);
							$devolveItens = true;
							//var_dump('passa produto '.$procuraItensCombo['CDPRODUTO'].' para dentro do combo');
						}
					}
				}
			}
			if(!empty($combo['PRODUTOS'])){
				$session = $this->util->getSessionVars(null);
				$precosCampanha = $this->promocaoService->formataPromocaoCombinada($session['CDFILIAL'], $CDCLIENTE, $CDCONSUMIDOR, $session['CDLOJA'], $combo['PRODUTOS'], $combo['CDPRODUTO'], null, null, true, $session['IDTIPCOBRA'], $combo['IDIMPPRODUTO'], $combo);
				// percorre os produtos retornados para atualizar os descontos/acrescimos no array principal
				foreach ($precosCampanha as $itemCampanha){
					foreach ($combo['PRODUTOS'] as $keyProdutoDoCombo => $produtoDoCombo){
						if($itemCampanha['ID'] == $produtoDoCombo['ID'] && $itemCampanha['CDPRODUTO'] == $produtoDoCombo['CDPRODUTO']){
							$vlrDesc = $itemCampanha['VRDESCCOMVEN'];
							$vlrAcre = $itemCampanha['VRACRCOMVEN'];
							if(strval($vlrDesc) != strval($itemCampanha['VRDESITVEND']) || strval($vlrDesc) != strval($itemCampanha['VRACRITVEND'])){
								$produtoDoCombo['VRDESITVEND'] = floatval(bcadd(strval($vlrDesc), strval($itemCampanha['VRDESITVEND']), '2'));
								$produtoDoCombo['VRACRITVEND'] = floatval(bcadd(strval($vlrAcre), strval($itemCampanha['VRACRITVEND']), '2'));
								$produtoDoCombo['PRITOTITEM'] = floatval(bcsub(strval($produtoDoCombo['PRITEM'] + $produtoDoCombo['VRPRECITEMCL'] + $produtoDoCombo['VRACRITVEND']), strval($produtoDoCombo['VRDESITVEND']), 2));
	
								//atualiza o array principal que vai ser retornado
								if(isset($produtoDoCombo['keyOriginal'])){
									$promocoes[$produtoDoCombo['keyOriginal']] = $produtoDoCombo;
									//var_dump('atualiza '.$produtoDoCombo['CDPRODUTO'].' fora do combo');
								} else {
									// se o combo for imprime sim então o desconto que iria para o filho deve ser dado ao pai
									if($promocoes[$keyCombo]['IDIMPPRODUTO'] == '1' ){
										$promocoes[$keyCombo]['VRDESITVEND'] += $produtoDoCombo['VRDESITVEND'];
										$promocoes[$keyCombo]['VRACRITVEND'] += $produtoDoCombo['VRACRITVEND'];
										//var_dump('atualiza o desconto que seria do produto '.$produtoDoCombo['CDPRODUTO'].' no combo');
									} else {
										$promocoes[$keyCombo]['PRODUTOS'][$keyProdutoDoCombo] = $produtoDoCombo;
										//var_dump('atualiza '.$produtoDoCombo['CDPRODUTO'].' dentro do combo');
									}
								}
							}
						}
					}
				}
			}
			if($devolveItens){
				/*
				não precisa mais voltar o item pra fora do combo porque que o array que vai ser retornado não foi alterado
				foreach ($combo['PRODUTOS'] as $i => $produtoTemp){
					foreach ($promocoes as $key => $procuraItensCombo){
						if(isset($produtoTemp['idComboProduto']) && isset($procuraItensCombo['idComboProduto'])){
							if(
								$produtoTemp['idComboProduto'] == $procuraItensCombo['idComboProduto'] &&
								$produtoTemp['CDPRODUTO'] == $procuraItensCombo['CDPRODUTO'] &&
								$produtoTemp['ID'] == $procuraItensCombo['ID']
							){
								var_dump('volta o produto '.$produtoTemp['CDPRODUTO'].' para fora');
								$promocoes[$key] = $produtoTemp;
								unset($combo['PRODUTOS'][$i]);
							}
						}
					}
				}
				*/
			}
		}
		return $promocoes;
	}

	private function utilCouvert($CDFILIAL, $NRCOMANDA, $NRVENDAREST, $IDUTILCOUVERT, $modoHabilitado = null){
		$mesasAgrupadas = $this->tableService->buscaMesasAgrupadas($NRCOMANDA, $NRVENDAREST);
		$mesasAgrupadas = $this->tableService->filtraMesasAgrupadas($modoHabilitado, $mesasAgrupadas, $NRVENDAREST, $NRCOMANDA);

		foreach ($mesasAgrupadas as &$mesa) {
			$params = array(
				'CDFILIAL'      => $CDFILIAL,
	            'NRCOMANDA'     => $mesa['NRCOMANDA'],
	            'NRVENDAREST'   => $mesa['NRVENDAREST'],
	            'IDUTILCOUVERT' => $IDUTILCOUVERT
			);
			$this->entityManager->getConnection()->executeQuery("SQL_ULTILIZA_COUVERT", $params);
		}
	}

    public function updateServiceTax($NRVENDAREST, $NRCOMANDA, $TOTALPRODS, $VRACRESCIMO, $TIPOGORJETA){
        $session = $this->util->getSessionVars(null);

        $VRCOMISVENDE = $VRACRESCIMO;
        $VRCOMISPOR = bcmul(strval($VRACRESCIMO) / strval($TOTALPRODS), '100', 3);

        $params = array(
            'CDFILIAL'     => $session['CDFILIAL'],
            'NRVENDAREST'  => $NRVENDAREST,
            'NRCOMANDA'    => $NRCOMANDA,
            'VRCOMISVENDE' => $VRCOMISVENDE,
            'VRCOMISPOR'   => $VRCOMISPOR
        );
        $this->entityManager->getConnection()->executeQuery("SQL_ATUALIZA_GORJETA", $params);

        $params = array(
            'CDFILIAL'    => $session['CDFILIAL'],
            'NRVENDAREST' => $NRVENDAREST
        );
        $this->entityManager->getConnection()->executeQuery("LIMPA_FIDELIDADE", $params);
    }

    public function alteraStatusComanda($cdfilial, $nrcomanda, $nrvendarest, $cdcaixa, $bloqueia){
    	$dados_caixa = $this->entityManager->getConnection()->fetchAssoc("SQL_DADOS_CAIXA", array($cdfilial, $cdcaixa));
			if($dados_caixa["IDBLOQCOMPARC"] === 'S' && $bloqueia == true){
				$params = array(
					'7',
					$cdfilial,
             		$nrcomanda,
             		$nrvendarest
				);
				$this->entityManager->getConnection()->fetchAssoc("SQL_ALTERA_STATUS_COMANDA", $params);
			}
    }

    public function searchConsumerOnline($searchValue, $CDFILIAL){
    	if($searchValue !== ''){
	        $params = array(
	            'SEARCH_VALUE' => $searchValue,
	            'CDFILIAL' => $CDFILIAL
	        );
	        $result = $this->extratocons->consultaConsumer($params['SEARCH_VALUE'], $params['CDFILIAL']);
	        if (!empty($result) && isset($result[0])){
	            $result = $result[0];
	            if (!array_key_exists(0, $result)){
	                $result = array(0 => $result);
	            }
	        }

	        foreach ($result as &$consumer){
	        	if(isset($consumer['CDFAMILISALD']))
	            	if (is_array($consumer['CDFAMILISALD'])) $consumer['CDFAMILISALD'] = null;
	            if(isset($consumer['NMFAMILISALD']))
	            	if (is_array($consumer['NMFAMILISALD'])) $consumer['NMFAMILISALD'] = null;
	            if(isset($consumer['CDFAMILISALD']) && isset($consumer['CDCLIENTE'])){
	            	$consumer['ID'] = $consumer['CDCLIENTE']."-".$consumer['CDCONSUMIDOR']."-".$consumer['CDFAMILISALD'];
	            }
	            $consumer['IDSITCONSUMI'] = '1';
	        }
	    } else $result = array();

        return $result;
    }

    public function verifyConsumerLocal($CDCONSUMIDOR, $consumerData){
		if(!$consumerData){
        	throw new Exception("Nenhum registro encontrado para o valor informado.", 1);
		}
		if(!isset($consumerData['NMCONSUMIDOR'])){
        	throw new Exception("Nenhum registro encontrado para o valor informado.", 1);
		}
    	$params = array(
            'CDCONSUMIDOR' => $consumerData['CDCONSUMIDOR'],
            'NMCONSUMIDOR' => $consumerData['NMCONSUMIDOR']
		);
		$results = $this->entityManager->getConnection()->fetchAll('SQL_GET_ESPECIFIC_CONSUMER', $params);
        if (!empty($results)){
			if(isset($results[0]['CDCLIENTE'])){
				$NMRAZSOCCLIE = $this->entityManager->getConnection()->fetchAssoc("SQL_VALIDA_CLIENTE", array($results[0]['CDCLIENTE']));
				if(!empty($NMRAZSOCCLIE['NMRAZSOCCLIE'])){
					$results[0]['NMRAZSOCCLIE'] = $NMRAZSOCCLIE['NMRAZSOCCLIE'];
				}
			}
			$results[0]['insert'] = false;
			return $results[0];
        } else if (!empty($consumerData)) {
			$this->consumidorService->registerConsumer($consumerData['CDCLIENTE'], $consumerData['CDCONSUMIDOR'], $consumerData['CDSENHACONSMD5'], $consumerData['CDIDCONSUMID'], $consumerData['NMCONSUMIDOR'], $consumerData['DSEMAILCONS'], $consumerData['NRCELULARCONS'], $consumerData['IDSITCONSUMI'], $consumerData['CDCCUSCLIE'], $consumerData['IDCONSUMIDOR'], $consumerData['IDATUCONSUMI'], $consumerData['IDTPVENDACONS'], $consumerData['IDIMPCPFCUPOM'], $consumerData['IDCADCONFLIBCON'], $consumerData['IDPERCONSPRODEX'], $consumerData['IDCONSUMIDOR'], $consumerData['IDTPSELMANHA'], $consumerData['IDTPSEALMOCO'], $consumerData['IDTPSELTARDE'], $consumerData['IDCRACHAMESTRE']);
			$NMRAZSOCCLIE = $this->entityManager->getConnection()->fetchAssoc("SQL_VALIDA_CLIENTE", array($consumerData['CDCLIENTE']));
			if(!empty($NMRAZSOCCLIE['NMRAZSOCCLIE'])){
				$consumerData['NMRAZSOCCLIE'] = $NMRAZSOCCLIE['NMRAZSOCCLIE'];
			}
			$consumerData['insert'] = true;
			return $consumerData;
        } else {
        	throw new Exception("Nenhum registro encontrado para o valor informado.", 1);
        }
        return $results;
	}

    /*public function verifyConsumerLocal($CDCONSUMIDOR, $consumerData){
    	$params = array(
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'NMCONSUMIDOR' => $consumerData['NMCONSUMIDOR']
        );
        $results = $this->entityManager->getConnection()->fetchAll('SQL_GET_ESPECIFIC_CONSUMER', $params);

        $consumerData['consumerCPF'] 			= $consumerData['NRCPFRESPCON'];
        $consumerData['consumerCod'] 			= $consumerData['CDCONSUMIDOR'];
        $consumerData['consumerPassAcess']  	= $consumerData['CDSENHACONSMD5'];
        $consumerData['conversion'] 			= null;
        $consumerData['consumerUpdateCPF']  	= $consumerData['IDATUCPFCONS'];
        $consumerData['consumerReleasedConsum'] = $consumerData['IDPERCONSPRODEX'];
        $consumerData['consumerPrint'] 			= $consumerData['IDIMPCPFCUPOM'];
        $consumerData['consumerVerifyBalance']  = $consumerData['IDVERSALDCON'];
        $consumerData['consumerName'] 			= $consumerData['NMCONSUMIDOR'];
        $consumerData['consumerGender'] 		= $consumerData['IDSEXOCONS'];
        $consumerData['consumerBirth'] 			= isset($dtFormated) ? $dtFormated : null;
        $consumerData['consumerRG'] 			= null; // $consumerData['NRRGCONSUMID'];
        $consumerData['consumerAdress'] 		= $consumerData['DSENDECONS'];
        $consumerData['consumerAdressNum'] 		= $consumerData['NRENDECONS'];
        $consumerData['NMBAIRRO'] 				= null; // $consumerData['NMBAIRCONS'];
        $consumerData['consumerCEP'] 			= $consumerData['NRCEPCONS'];
        $consumerData['consumerResPhone'] 		= $consumerData['NRTELECONS'];
        $consumerData['consumerComPhone'] 		= null; // $consumerData['NRTELE2CONS'];
        $consumerData['consumerCellPhone'] 		= $consumerData['NRCELULARCONS'];
        $consumerData['consumerEmail'] 			= !empty($consumerData['DSEMAILCONS']) ? $consumerData['DSEMAILCONS'] : null;
        $consumerData['CDTIPOVENDA'] 			= $consumerData['IDTPVENDACONS'];
        $consumerData['consumerId'] 			= $consumerData['CDIDCONSUMID'];
        $consumerData['consumerCodAcessExt'] 	= !empty($consumerData['CDEXCONSUMID']) ? $consumerData['CDEXCONSUMID'] : null;
        $consumerData['consumerCodAcess'] 		= $consumerData['NMLOGINCONS'];
        $consumerData['CDBAIRRO'] 				= $consumerData['IDTPVENDACONS'];

        if (!empty($results) && isset($results[0])){
        	if(is_array($results)){
        		$arr = array();
        		$arr['consumerCPF'] 			= $results[0]['NRCPFRESPCON'];
		        $arr['consumerCod'] 			= $results[0]['CDCONSUMIDOR'];
		        $arr['consumerPassAcess'] 		= $results[0]['CDSENHACONSMD5'];
		        $arr['conversion'] 				= null;
		        $arr['consumerUpdateCPF'] 		= $results[0]['IDATUCPFCONS'];
		        $arr['consumerReleasedConsum']  = $results[0]['IDPERCONSPRODEX'];
		        $arr['consumerPrint'] 			= $results[0]['IDIMPCPFCUPOM'];
		        $arr['consumerVerifyBalance'] 	= $results[0]['IDVERSALDCON'];
		        $arr['consumerName'] 			= $results[0]['NMCONSUMIDOR'];
		        $arr['consumerGender'] 			= $results[0]['IDSEXOCONS'];
		        $arr['consumerBirth'] 			= isset($dtFormated) ? $dtFormated : null;
		        $arr['consumerRG'] 				= null; // $consumerData['NRRGCONSUMID'];
		        $arr['consumerAdress'] 			= $results[0]['DSENDECONS'];
		        $arr['consumerAdressNum'] 		= $results[0]['NRENDECONS'];
		        $arr['NMBAIRRO'] 				= null; // $consumerData['NMBAIRCONS'];
		        $arr['consumerCEP'] 			= $results[0]['NRCEPCONS'];
		        $arr['consumerResPhone'] 		= $results[0]['NRTELECONS'];
		        $arr['consumerComPhone'] 		= null; // $consumerData['NRTELE2CONS'];
		        $arr['consumerCellPhone'] 		= $results[0]['NRCELULARCONS'];
		        $arr['consumerEmail'] 			= !empty($results[0]['DSEMAILCONS']) ? $results[0]['DSEMAILCONS'] : null;
		        $arr['CDTIPOVENDA'] 			= $results[0]['IDTPVENDACONS'];
		        $arr['consumerId'] 				= $results[0]['CDIDCONSUMID'];
		        $arr['consumerCodAcessExt'] 	= !empty($results[0]['CDEXCONSUMID']) ? $results[0]['CDEXCONSUMID'] : null;
		        $arr['consumerCodAcess'] 		= $results[0]['NMLOGINCONS'];
		        $arr['CDBAIRRO'] 				= $results[0]['IDTPVENDACONS'];
		        foreach ($arr as $key => $val) {
		        	$results[0][$key] = $val;
		        }
        		$this->consumidorService->addConsumer($results[0], true);
        	}
        } else if (!empty($consumerData) && isset($consumerData)) {
        	$this->consumidorService->addConsumer($consumerData, true);
        } else {
        	throw new Exception("Nenhum registro encontrado para o valor informado.", 1);

        }
        return $results;
    }*/
}
