<?php
namespace Odhen\API\Service;

use \SimpleXMLElement;

class FidelidadeUDS {
	protected $sessionHandler;
	
	public function __construct(
		\Helpers\Environment $sessionHandler
	){
        $this->sessionHandler = $sessionHandler;
	}
    
    public function consultaFidelidadeUDS($params){
        try{
            $result = self::chamadaFidelidadeUDS('getSaldo', $params);
            $result['erro'] = false;
        } catch (\Exception $e) {
            $result['success'] = false;
            $result['erro'] = true;
            $result['message'] = $e->getMessage();
            $result['fidelidade']='UDS';
            $result['bloqueado']=false;
            $result['clienteFidelidade']=true;
            $result['saldoDisp']= 0;
            $result['cpfCli']=isset($params['cpfCli'])?$params['cpfCli']:'';
            $result['nomeCompleto']='';
            $result['userInfo']['balance'] = 0;
            $result['userInfo']['cpf'] = isset($params['cpfCli'])?$params['cpfCli']:'';
            $result['userInfo']['name'] = '';
            $result['fidelityDetails'] = $result;
        }
        return $result;
    }

    public function transmiteVendaUDS($vendaJSON){
        $result = self::chamadaFidelidadeUDS('sendSale', $vendaJSON);
        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }
    public function transmitePrePedido($pedidoJSON){
        $result = self::chamadaFidelidadeUDS('sendOrder', $pedidoJSON);
        return $result;
    }

    public function deletaPrePedido($pedidoJSON){
        $result = self::chamadaFidelidadeUDS('deleteOrder', $pedidoJSON);
        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    public function cancelaFidelidadeUDS($vendaXML){
        $result = $this::chamadaFidelidadeUDS('cancelSale', $vendaXML);
        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    public function chamadaFidelidadeUDS($rota, $params){
        self::LogUDS('chamadaFidelidadeUDS');
        $URL = $this->sessionHandler->getUserInfo()['CDURLWSFIDUDS'];
        $URL = rtrim($URL, '/') . "/" .  $rota;
        $curl = curl_init();
        self::LogUDS($URL);
        curl_setopt($curl, CURLOPT_URL, $URL);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/xml'));

        if ($rota == 'getSaldo'){
            $params = self::ArrayToXml($params);
        } else if ($rota == 'cancelSale' || $rota == 'deleteOrder' || $rota == 'sendSale'){
            $params = self::ArrayToXml(json_decode($params, true));
        } else if ($rota == 'sendOrder'){
            $params = self::ArrayToXml($params);
        }
        self::LogUDS('PARAMETERS: '. preg_replace( '/[\n,  ]/', "", $params ));
        curl_setopt($curl, CURLOPT_POSTFIELDS, $params);

        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_ENCODING,'');
        curl_setopt($curl, CURLOPT_MAXREDIRS,10);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_HTTP_VERSION , CURL_HTTP_VERSION_1_1);

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 14);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 14);

        $curl_response = curl_exec($curl);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        self::LogUDS('httpcode: '.$httpcode);
        if ($httpcode ==='404' || $httpcode === 404){
            self::LogUDS('Erro 404');
            throw new \Exception("<br>Erro 404 Page Not Found.");
        } else {
            self::LogUDS('RESPONSE: '.$curl_response);
        }
        if ($err){
            self::LogUDS('err: '.$err);
            self::LogUDS('errmsg: '.$errmsg);
            if ($err === 28){
                throw new \Exception("<br>N&atilde;o foi poss&iacute;vel estabelecer uma conex&atilde;o com o servidor.");
            }
            else {
                throw new \Exception("<br>Ocorreu um erro na chamada com o servidor:<br><br>" . $errmsg);
            }
        }
        if (empty($curl_response)){
            self::LogUDS('empty curl_response');
            throw new \Exception("<br>N&atilde;o houve retorno do servidor.");
        }
        $result = self::XmlToJson($curl_response);
        $result = json_decode($result, true);
        if (empty($result)) throw new \Exception($curl_response);
        $result['fidelidade']='UDS';
        if (array_key_exists('success', $result)){
            self::LogUDS('array_key_exists: success');
            if($result['success'] === true || $result['success'] === 'true'){
                $result['success'] = true;
                if ($rota == 'getSaldo' ){
                    $result['userInfo']['balance'] = $result['saldoDisp'];
                    $result['userInfo']['cpf'] = $result['cpfCli'];
                    $result['userInfo']['name'] = $result['nomeCompleto'];
                    if (array_key_exists('bloqueado', $result)){
                        if($result['bloqueado'] === true || $result['bloqueado'] === 'true'){
                            $result['bloqueado'] = true;
                            $result['saldoDisp'] = 0;
                            $result['userInfo']['balance'] = 0;
                        } else {
                            $result['bloqueado'] = false;
                        }
                    } else {
                        $result['bloqueado'] = false;
                    }

                    if (array_key_exists('clienteFidelidade', $result)){
                        if($result['clienteFidelidade'] === true || $result['clienteFidelidade'] === 'true'){
                            $result['clienteFidelidade'] = true;
                        } else {
                            $result['clienteFidelidade'] = false;
                        }
                    } else {
                        $result['clienteFidelidade'] = true;
                    }
                }
                self::LogUDS('');
                return $result;
            } else {
                if(isset($result['message'])){
                    self::LogUDS('message: '.$result['message']);
                    self::LogUDS('');
                    throw new \Exception($result['message']);
                } else if(isset($result['error'])){
                    self::LogUDS('error: '.$result['error']);
                    self::LogUDS('');
                    throw new \Exception($result['error']);
                } else {
                    self::LogUDS('Erro na consulta fidelidade');
                    self::LogUDS('');
                    throw new \Exception("Erro na consulta fidelidade");
                }
            }
        } else {
            self::LogUDS('array_key_NOT_exists: success');
            if(isset($result['error'])){
                self::LogUDS('error: '.$result['error']);
                self::LogUDS('');
                throw new \Exception($result['error']);
            } else if(isset($result['message'])){
                self::LogUDS('message: '.$result['message']);
                self::LogUDS('');
                throw new \Exception($result['message']);
            } else {
                self::LogUDS(print_r($result,true));
                self::LogUDS('');
                throw new \Exception("Erro na consulta fidelidade");
            }
        }
    }

    public function XmlToJson ($xml) {
        self::LogUDS('XmlToJson');
        $xml = str_replace(array("\n", "\r", "\t"), '', $xml);
        $xml = trim(str_replace('"', "'", $xml));
        $simpleXml = simplexml_load_string($xml);
        $json = json_encode($simpleXml);
        self::LogUDS($json);
        return $json;
    }

    function ArrayToXml($array, $xml = null){
        $boprimeira = false;
        if($xml == null){
            $xml = new SimpleXMLElement('<root/>');
            $boprimeira = true;
            self::LogUDS('ArrayToXml');
        }        
        foreach ($array as $key => $value) {
            if(is_int($key) && is_array($value)){
                self::ArrayToXml($value, $xml);
            } else {
                if(is_int($key)){
                    if($key===0){
                        $key = '0';
                    } else {
                        $key = strval($key);
                    }
                }
                if(is_bool($key)){
                    if($key){
                        $key = 'true';
                    }else {
                        $key = 'false';
                    }
                }
                if(is_array($value)){
                    $label = $xml->addChild($key);
                    self::ArrayToXml($value, $label);
                }
                else {
                    $xml->addChild($key, $value);
                }
            }
        }
        if($boprimeira){
            //self::LogUDS($xml->asXml());
        }
        return $xml->asXml();
    }

    public function LogUDS($texto){
        return self::Log('UDS',$texto);
    }

    public function Log($tipo,$texto){
        if($tipo=='UDS'){
            $tipo ='fidelidadeUDS';
        }
        $paths = ['LOG/', $tipo.'/', (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('m-Y')];
        $logVendaPath = dirname(__DIR__) . '/../../../../../../../';
        foreach ($paths as $path){
            $logVendaPath = $logVendaPath . $path;
            if (!is_dir($logVendaPath)){
                mkdir($logVendaPath, 0700);
            }
        }
        $logVendaPath = $logVendaPath .'/'. (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('d-m-Y') . '.txt';
        $horaAtual = (new \DateTime('NOW', new \DateTimeZone('America/Bahia')))->format('d/m/Y H:i:s');
        file_put_contents($logVendaPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
    }
}